/*****************************************************************************
 * 21835_FIRA_Throughput_L1.c
 *****************************************************************************/

#include "adi_initialize.h"
#include "21835_FIRA_Throughput_L1.h"

#include <sys/platform.h>
#include <stdio.h>
#include <time.h>
#include <stdlib.h>

/** 
 * If you want to use command program arguments, then place them in the following string. 
 */
char __argv_string[] = "";

volatile clock_t clock_start;
volatile clock_t clock_stop;
volatile clock_t cycles;
volatile clock_t cycles_offset;

//#pragma section("seg_l1_block0")
__attribute__((section(".L1.data")))
//__attribute__((section(".L2.data")))
//__attribute__((section(".L3.data")))
int FIRA_TCB[FIR_TCB_SIZE];

//#pragma section("seg_l1_block1")
__attribute__((section(".L1.data")))
//__attribute__((section(".L2.data")))
//__attribute__((section(".L3.data")))
float InputBuff[MAX_TAP_LENGTH+MAX_WINDOW_SIZE-1]=
{
	#include "fir_input_taps_64_window_1024.dat"
};

//#pragma section("seg_l1_block2")
//__attribute__((section(".L1.text")))
__attribute__((section(".L1.data")))
//__attribute__((section(".L2.data")))
//__attribute__((section(".L3.data")))
float CoeffBuff[MAX_TAP_LENGTH]=
{
	#include "fir_coeffs_64.dat"
};

//#pragma section("seg_l1_block3")
//__attribute__((section(".L1.text")))
__attribute__((section(".L1.data")))
//__attribute__((section(".L2.data")))
//__attribute__((section(".L3.data")))
float OutputBuff[MAX_WINDOW_SIZE];

float ExpectedOutputBuff[MAX_WINDOW_SIZE] =
{
//#include "fir_output_taps_64_window_1024.dat"
};
void FIRA_Init(uint32_t uiFIR_GCTL, uint32_t* uiFIR_CHNPTR)
{
	*pREG_FIR0_CHNPTR=(uint32_t)uiFIR_CHNPTR;
	clock_start=clock();
	*pREG_FIR0_CTL1=uiFIR_GCTL;
}

uint32_t i = 0;

float Find_Max_Diff(float* uiBuff_actual, float* uiBuff_expected, unsigned int uiCount)
{
	volatile uint32_t i; float temp, temp1;
	float max_diff=0; float diff;
	float* pt = uiBuff_actual; float*  pt1 = uiBuff_expected;

	for(i=0;i<uiCount;i++)
	{
		temp=*pt++; temp1=*pt1++;

		if(temp>temp1)
			diff = temp-temp1;
		else
			diff = temp1-temp;

		if(temp1!=0.0&&diff>max_diff)
			max_diff=diff/temp1;
	}

	return max_diff;
}

#define MAX_DIFF_LIMIT 0.001

/* Macro for checking Maximum Diff exceeding the limit */
#define CHECK_MAX_DIFF(max_diff) \
        if(max_diff > MAX_DIFF_LIMIT)\
		{\
            printf("CHECK_MAX_DIFF failed at line %d of file %s \n",__LINE__,__FILE__);\
            exit(-2);\
        }

uint32_t TranslateAddr (uint32_t  MemAddr)
{
    /* The TCB addresses have been divided by 4. Adjust them so the common TranslateAddr routine works */
#if defined(__ADSPSHARC__) && (__ADSPSHARC__ >= 0x200) || (defined(__ADSPSC835_FAMILY__) && defined(__ADSPSHARCFX__))
    MemAddr = adi_rtl_internal_to_system_addr(MemAddr, 1);
#endif

    return (MemAddr >> 2u);
}

int main(int argc, char *argv[])
{
	/**
	 * Initialize managed drivers and/or services that have been added to 
	 * the project.
	 * @return zero on success 
	 */
	adi_initComponents();

#if 1
	clock_start=clock();
	clock_stop=clock();
	cycles_offset=clock_stop-clock_start;

	printf("%d\n",cycles_offset);
#endif



    float i = ExpectedOutputBuff[1];
#if 1
	uint32_t w; /* Window */;
	uint32_t t; /* Taps */

	FILE * fpout;

	fpout = fopen("Results(Cycles).csv","w+");

	for(w=1;w<=MAX_WINDOW_SIZE;w++)
	{
		for(t=1;t<=MAX_TAP_LENGTH;t++)
		{
			//Initialize FIR TCBs for both the channels
			FIRA_TCB[0]=0;//((int)(FIRA_TCB2+12)>>2)|MP_OFFSET;	//CP
			FIRA_TCB[1]=t;	//CL
			FIRA_TCB[2]=1;	//CM
			FIRA_TCB[3]= (void *)TranslateAddr((uint32_t)CoeffBuff);	//CI
			FIRA_TCB[4]=(void *)TranslateAddr((uint32_t)OutputBuff);	//OB
			FIRA_TCB[5]=w;	//OL
			FIRA_TCB[6]=1;	//OM
			FIRA_TCB[7]=(void *)TranslateAddr((uint32_t)OutputBuff);	//OI
			FIRA_TCB[8]=(void *)TranslateAddr((uint32_t)InputBuff);	//IB
			FIRA_TCB[9]=t+w-1;	//IL
			FIRA_TCB[10]=1;	//IM
			FIRA_TCB[11]=(void *)TranslateAddr((uint32_t)InputBuff);	//II
			FIRA_TCB[12]=(t-1)|(w-1)<<BITP_FIR_CTL2_WINDOW;	//FIRCTL2

			//Disable accelerator
			*pREG_FIR0_CTL1 &= ~(BITM_FIR_CTL1_EN);

			//Start the IIR accelerator
			FIRA_Init(BITM_FIR_CTL1_EN|BITM_FIR_CTL1_DMAEN|(CHANNEL_NO-1)<<BITP_FIR_CTL1_CH|BITM_FIR_CTL1_BURSTEN|BITM_FIR_CTL1_PFB_EN,(uint32_t *)(void *)TranslateAddr((uint32_t)((int)(FIRA_TCB+12))));

			//wait till all channels are done
			while((*pREG_FIR0_DMASTAT & BITM_FIR_DMASTAT_ACDONE) == 0x00000000);

			clock_stop=clock();
			cycles=clock_stop-clock_start;

#if 0
			volatile float max_diff;
			max_diff=Find_Max_Diff(OutputBuff, ExpectedOutputBuff,1024);
			CHECK_MAX_DIFF(max_diff)
#endif
			//printf("%d\n",cycles);
			fprintf(fpout,"%d,",cycles-cycles_offset);
	    }

		fprintf(fpout,"\n");
	}
	fclose(fpout);
#endif

	return 0;
}

