/*****************************************************************************
 * 21835_IIRA_Throughput_L1.c
 *****************************************************************************/

#include <sys/platform.h>
#include "adi_initialize.h"
#include "21835_IIRA_Throughput_L1.h"

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <time.h>

volatile clock_t clock_start;
volatile clock_t clock_stop;
volatile clock_t cycles;
volatile clock_t cycles_offset;

//#pragma section("seg_l1_block0")
__attribute__((section(".L1.text")))
uint32_t IIRA_TCB[IIR_TCB_SIZE];

//#pragma section("seg_l1_block1")
__attribute__((section(".L1.data")))
float OutputBuff[MAX_WINDOW_SIZE];

//#pragma section("seg_l1_block1")
__attribute__((section(".L1.data")))
float InputBuff[MAX_WINDOW_SIZE] =
{
		//#include "iir_input_window_1024.dat"
};

//#pragma section("seg_l1_block1")
__attribute__((section(".L1.data")))
float CoeffBuff[7*MAX_BIQUADS] =
{
		//#include "iir_coeffs_biquads_12.dat"
};

float ExpectedOutputBuff[MAX_WINDOW_SIZE] =
{
		//#include "iir_output_biquads_12_window_1024.dat"
};

float SOS_Gain1[1]=
{
	//#include "iir_sos_gain_biquads_12.dat"
};

void IIRA_Init(uint32_t uiIIR_GCTL, uint32_t* uiIIR_CHNPTR)
{
	*pREG_IIR0_CHNPTR=(uint32_t)uiIIR_CHNPTR;
	clock_start=clock();
	*pREG_IIR0_CTL1=uiIIR_GCTL;
}
/** 
 * If you want to use command program arguments, then place them in the following string. 
 */
char __argv_string[] = "";

float Find_Max_Diff(float* uiBuff_actual, float* uiBuff_expected, unsigned int uiCount)
{
	volatile uint32_t i; float temp, temp1;
	float max_diff=0; float diff;
	float* pt = uiBuff_actual; float*  pt1 = uiBuff_expected;

	for(i=0;i<uiCount;i++)
	{
		temp=*pt++; temp1=*pt1++;

		if(temp>temp1)
			diff = temp-temp1;
		else
			diff = temp1-temp;

		if(temp1!=0.0&&diff>max_diff)
			max_diff=diff/temp1;
	}

	return max_diff;
}

#define MAX_DIFF_LIMIT 0.001

/* Macro for checking Maximum Diff exceeding the limit */
#define CHECK_MAX_DIFF(max_diff) \
        if(max_diff > MAX_DIFF_LIMIT)\
		{\
            printf("CHECK_MAX_DIFF failed at line %d of file %s \n",__LINE__,__FILE__);\
            exit(-2);\
        }

uint32_t TranslateAddr (uint32_t  MemAddr)
{
    /* The TCB addresses have been divided by 4. Adjust them so the common TranslateAddr routine works */
#if defined(__ADSPSHARC__) && (__ADSPSHARC__ >= 0x200) || (defined(__ADSPSC835_FAMILY__) && defined(__ADSPSHARCFX__))
    MemAddr = adi_rtl_internal_to_system_addr(MemAddr, 1);
#endif

    return (MemAddr >> 2u);
}

void ScaleOutput(float *Buffer, uint32_t count, float gain)
{
	volatile uint32_t i = 0;
	for(i=0;i<count;i++)
	{
		Buffer[i] *= gain;
	}
}

int main(int argc, char *argv[])
{
	/**
	 * Initialize managed drivers and/or services that have been added to 
	 * the project.
	 * @return zero on success 
	 */
	adi_initComponents();

#if 1
	clock_start=clock();
	clock_stop=clock();
	cycles_offset=clock_stop-clock_start;

	printf("%d\n",cycles_offset);
#endif
	
	uint32_t w; /* Window */;
	uint32_t b;

	FILE * fpout;
	fpout = fopen("Results(cycles).csv","w+");

	for(w=1;w<=MAX_WINDOW_SIZE;w++)
	{
		for(b=1;b<=MAX_BIQUADS;b++)
		{
			//Initialize IIR TCBs
			IIRA_TCB[0]=(uint32_t)(void *)TranslateAddr((uint32_t)((int)(IIRA_TCB+12)));	//CP
			IIRA_TCB[1]=b*7;	//CL
			IIRA_TCB[2]=1;	//CM
			IIRA_TCB[3]= (void *)TranslateAddr((uint32_t)CoeffBuff);  //CI
			IIRA_TCB[4]= (void *)TranslateAddr((uint32_t)OutputBuff); //OB
			IIRA_TCB[5]=w;	//OL
			IIRA_TCB[6]=1;	//OM
			IIRA_TCB[7]=(void *)TranslateAddr((uint32_t)OutputBuff); //OI
			IIRA_TCB[8]=(void *)TranslateAddr((uint32_t)InputBuff);  //IB
			IIRA_TCB[9]=w;	//IL
			IIRA_TCB[10]=1;	//IM
			IIRA_TCB[11]=(void *)TranslateAddr((uint32_t)InputBuff); //II
			IIRA_TCB[12]=(b-1)|((w-1)<<BITP_IIR_CTL2_WINDOW);	//IIRCTL2

			//Disable accelerator
			*pREG_IIR0_CTL1 &= ~BITM_IIR_CTL1_EN;

			//Start the IIR accelerator
			IIRA_Init(BITM_IIR_CTL1_EN|BITM_IIR_CTL1_DMAEN|((CHANNEL_NO-1)<<BITP_IIR_CTL1_CH)|BITM_IIR_CTL1_BURSTEN|BITM_IIR_CTL1_SS,(uint32_t *)(void *)TranslateAddr((uint32_t)((int)(IIRA_TCB+12))));

			//wait till all channels are done
			while((*pREG_IIR0_DMASTAT & BITM_IIR_DMASTAT_ACDONE) == 0x00000000);

			//wait for Save state to be complete
			while((*pREG_IIR0_DMASTAT & (BITM_IIR_DMASTAT_CPLD|BITM_IIR_DMASTAT_SVDK)) != 0x0);

			clock_stop=clock();
			cycles=clock_stop-clock_start;

#if 0
			volatile float max_diff;
			ScaleOutput(OutputBuff,1024,SOS_Gain1[0]);
			max_diff=Find_Max_Diff(OutputBuff, ExpectedOutputBuff,1024);
			CHECK_MAX_DIFF(max_diff)
#endif

			//printf("%d\n",cycles);
			fprintf(fpout,"%d,",cycles-cycles_offset);

		}

		fprintf(fpout,"\n");
	}

	fclose(fpout);

	return 0;
}

