/*!
 *****************************************************************************
 * @file:    adi_fir_def_legacy_v3.h
 * @brief:   FIR Device Driver definition for Legacy Mode
 * @version: $Revision: 37632 $
 * @date:    $Date: 2019-10-09 12:35:37 +0530 (Wed, 09 Oct 2019) $
 *****************************************************************************
Copyright(c) 2022-2023 Analog Devices, Inc. All Rights Reserved.
This software is proprietary.  By using this software you agree
to the terms of the associated Analog Devices License Agreement.
*********************************************************************************
     File         :   adi_fir_def_legacy_v3.h
     Description  :   Static function and structure definition needed by the device driver
                      for managing the FIR  device for ADSP-21569 processors.
================================================================================ */

#ifndef __ADI_FIR_LEGACY_DEF_V3_H__
#define __ADI_FIR_LEGACY_DEF_V3_H__




/*==========  D E F I N E S  ==========*/


#define FIR_MAX_NUM_CHANNELS          (32u)
#define FIR_MAX_NUM_TAPS_PER_CHANNEL  (4096u)

#define ADI_FIR_NUM_OF_CALLBACKS       (1u)
#define ADI_FIR_NUM_OF_CALLBACK_PARAMS (1u)
#define ADI_FIR_MASTER_CALLBACK_INDEX  (0u)

#define ADI_FIR_CFG_BURSTEN        (1ul << BITP_FIR_CTL1_BURSTEN)

#define ADI_FIR_CFG_BURST16_DIS     (0u)

#if (ADI_FIR_FIXED_POINT_MODE == 1u)
#define ADI_FIR_CFG_FIXED_POINT    ((1ul << BITP_FIR_CTL1_FXD) | (ADI_FIR_CFG_FIXED_INPUT_FORMAT << BITP_FIR_CTL1_TC))
#else
#define ADI_FIR_CFG_FIXED_POINT    (0u)
#endif /* (ADI_FIR_FIXED_POINT_MODE == 1u) */

/* Default FIRCTL1 Configuration */
#define FIR_DEF_FIRCTL1     (  (((uint32_t)ADI_FIR_CFG_ROUNDING_MODE) << BITP_FIR_CTL1_RND)                    \
                             | (((uint32_t)ADI_FIR_CFG_FIXED_POINT))                                           \
							 | (((uint32_t)ADI_FIR_CFG_BURSTEN))                                               \
							 | (((uint32_t)ADI_FIR_CFG_BURST16_DIS))                                           \
                            )

/* FIR_CTL1 Register macros */
#define FIR_FIRCTL1_EN_ENABLE                     (BITM_FIR_CTL1_EN)
#define FIR_FIRCTL1_DMAEN_ENABLE                  (BITM_FIR_CTL1_DMAEN)

/* TCB Info Structure */
typedef struct
{
    /* Chain Pointer */
    void*          pNextTCB;

    /* Coefficient Buffer */
    uint32_t       nCoeffCount;
    int32_t        nCoeffModify;
    void*          pCoeffIndex;

    /* Output Circular Buffer */
    void*          pOutputBase;
    uint32_t       nOutputCount;
    int32_t        nOutputModify;
    void* volatile pOutputIndex;

    /* Input Circular Buffer */
    void*          pInputBase;
    uint32_t       nInputCount;
    int32_t        nInputModify;
    void* volatile pInputIndex;

    /* Registers */
    uint32_t     nFirControl2;
} ADI_FIR_TCB_INFO;

/* Task Info structure */
typedef struct ADI_FIR_TASK_INFO_T
{
	struct ADI_FIR_TASK_INFO_T     *pNext;
	ADI_FIR_TCB_INFO               *pTCBChain;
	uint32_t                       nNumChannels;
	uint32_t                       nFIRCTL1;
	ADI_FIR_TASK_STATE             eTaskState;
	ADI_FIR_DEV_HANDLE             *pDevice;
	ADI_CORE_ID                    eCoreId;
	/* The Reserved bytes are added to make sure that Task Info and TCB Chain are on two different cache lines */
	uint8_t                        RESERVED[ADI_CACHE_LINE_LENGTH];
}ADI_FIR_TASK_INFO;

#if defined(ADI_USE_DEBUG_DATA)
/* It is advised not to use debug Data while running infinite loop programs as the indexes for these
 * structures/data may go out of bound and cause unexpected results in the programs
 */
typedef struct
{
	int FirIntCnt[2];
	int SWIntCnt[2];
	int QueueTracker[30];
	int QueueTrackerIndex;
}ADI_FIR_DEBUG_DATA;
#endif /* defined(ADI_USE_DEBUG_DATA) */

/* Queue Info Structure */
typedef struct
{
	ADI_FIR_TASK_INFO *pActiveTask;
 	ADI_FIR_TASK_INFO *pLastTaskAdded;
    ADI_FIR_TASK_INFO *pTaskCompleted;
#if defined(ADI_USE_DEBUG_DATA)
    volatile ADI_FIR_DEBUG_DATA *pFirDebugData;
#endif/* ADI_USE_DEBUG_DATA */
}ADI_FIR_QUEUE_INFO;

/* Device Info structure */
typedef struct
{
	ADI_FIR_DEV_STATE              eDevState;
	ADI_FIR_QUEUE_INFO             *pQueue;
	ADI_CALLBACK                   pfFIRCallback[ADI_FIR_NUM_OF_CALLBACKS];
	void *                         pFIRCBParam[ADI_FIR_NUM_OF_CALLBACK_PARAMS];
	ADI_FIR_TypeDef *const         pFIRRegs;           /* Base Address of FIR instance registers */
	uint32_t                       nFirDoneIntId;          /* SEC FIR DMA Done Interrupt ID */
}ADI_FIR_DEV_INFO;


#endif /* __ADI_FIR_LEGACY_DEF_V3_H__ */
