/*!
 *****************************************************************************
 * @file:    adi_iir_def_v3.h
 * @brief:   Common  definition file for IIR Device
 * @version: $Revision: 64994 $
 * @date:    $Date: 2020-09-24 12:17:24 -0400 (Thu, 24 Sep 2020) $
 *****************************************************************************
Copyright(c) 2022-2023 Analog Devices, Inc. All Rights Reserved.
This software is proprietary.  By using this software you agree
to the terms of the associated Analog Devices License Agreement.
*********************************************************************************
     File         :   adi_iir_def_v3.h
     Description  :   Common file for Definitions of structure for IIR device for ADSP-SC83x processors
================================================================================ */

#ifndef __ADI_IIR_DEF_V3_H__
#define __ADI_IIR_DEF_V3_H__

#include "sys/platform.h"
#include <drivers/iir/adi_iir_config_SC8xx.h>

/*==========  I N C L U D E  ==========*/
#include <adi_osal.h>
#if defined(__ADSPSC835_FAMILY__)
#include <ADSP-SC83x_device.h>
#else
#error "This processor is not supported"
#endif


#if (defined(__ADSPSC835_FAMILY__))

#define IIR_NUM_DEVICES_PER_SHARC    (4)

#define IIR_NUM_DEVICES    (IIR_NUM_DEVICES_PER_SHARC*__NUM_SHARC_CORES__)

#else

#error "This processor is not supported"

#endif /* (defined(__ADSPSC835_FAMILY__)) */

#if ((ADI_IIR_CFG_ACCELERATOR_MODE == ADI_IIR_ACCELERATOR_MODE_LEGACY))

/* IIR Driver implementation source for legacy mode */
#include "adi_iir_def_legacy_v3.h"

#elif ((ADI_IIR_CFG_ACCELERATOR_MODE == ADI_IIR_ACCELERATOR_MODE_ACM))

/* IIR Driver implementation source for ACM mode */
#include "adi_iir_def_acm_v3.h"

#else

#error "Selected Configuration/Processor not Supported"

#endif

#endif /* __ADI_IIR_DEF_V3_H__ */
