/*********************************************************************************
Copyright(c) 2021 Analog Devices, Inc. All Rights Reserved.
This software is proprietary and confidential. By using this software you agree
to the terms of the associated Analog Devices License Agreement.
*********************************************************************************/

/*****************************************************************************
 * Audio_Talkthrough_I2S.c
 *****************************************************************************/

#include <sys/platform.h>
#include "adi_initialize.h"
#include <stdio.h>
#include <stdlib.h>
#include <sys/adi_core.h>
#include "adi_initialize.h"
#include <services/int/adi_int.h>
#include <drivers/sport/adi_sport.h>
#include <services/spu/adi_spu.h>
#include <drivers/twi/adi_twi.h>
#include "ADAU_1962Common.h"
#include "ADAU_1979Common.h"
#include "math.h"
#include "sru.h"
#include "Audio_Talkthrough_I2S.h"
#include "Processing.h"
#include <services\fpll\adi_fpll.h>

struct Config_Table
{
	short Reg_Add;
	char  Value;
};

char Config_read_DAC[28];
char Config_read_ADC[16];
int count;
struct Config_Table Config_array_DAC[28] = {
		   	    {     ADAU1962_PDN_CTRL_1,		0x00},
		   	    {     ADAU1962_PDN_CTRL_2,	    0xff},
		   	    {     ADAU1962_PDN_CTRL_3,	    0x0f},
		   	    {     ADAU1962_DAC_CTRL0,		0x01},
		   	    {     ADAU1962_DAC_CTRL1,		0x01},
		   	    {     ADAU1962_DAC_CTRL2,		0x00},
		   	    {     ADAU1962_DAC_MUTE1,	    0x0},
		   	    {     ADAU1962_DAC_MUTE2,	    0x00},
		   	    {     ADAU1962_MSTR_VOL,		0x00},
			    {     ADAU1962_DAC1_VOL,	    0x00},
			    {     ADAU1962_DAC2_VOL,		0x00},
				{     ADAU1962_DAC3_VOL,		0x00},
				{     ADAU1962_DAC4_VOL,		0x00},
				{     ADAU1962_DAC5_VOL,		0x00},
				{     ADAU1962_DAC6_VOL,		0x00},
				{     ADAU1962_DAC7_VOL,		0x00},
				{     ADAU1962_DAC8_VOL,	    0x00},
				{     ADAU1962_DAC9_VOL,		0x00},
				{     ADAU1962_DAC10_VOL,		0x00},
				{     ADAU1962_DAC11_VOL,		0x00},
				{     ADAU1962_DAC12_VOL,		0x00},
				{     ADAU1962_PAD_STRGTH,		0x00},
				{     ADAU1962_DAC_PWR1,		0xaa},
				{     ADAU1962_DAC_PWR2,		0xaa},
				{     ADAU1962_DAC_PWR3,		0xaa},
				{     ADAU1962_PDN_CTRL_2,	    0x00},
				{     ADAU1962_PDN_CTRL_3,	    0x00},
				{     ADAU1962_DAC_CTRL0,		0x00}

};

struct Config_Table Config_array_ADC[16] = {
		 {ADAU1979_REG_BOOST			,	0x00},
		 {ADAU1979_REG_MICBIAS			,	0x00},
		 {ADAU1979_REG_BLOCK_POWER_SAI	,	0x30},
		 {ADAU1979_REG_SAI_CTRL0		,	0x02},
		 {ADAU1979_REG_SAI_CTRL1		,	0x00},
		 {ADAU1979_REG_CMAP12			,	0x01},
		 {ADAU1979_REG_CMAP34			,	0x00},
		 {ADAU1979_REG_SAI_OVERTEMP		,	0x30},
		 {ADAU1979_REG_POST_ADC_GAIN1 	,	0xA0},
		 {ADAU1979_REG_POST_ADC_GAIN2 	,	0xA0},
		 {ADAU1979_REG_POST_ADC_GAIN3 	,	0xA0},
		 {ADAU1979_REG_POST_ADC_GAIN4 	,	0xA0},
		 {ADAU1979_REG_ADC_CLIP			,	0x00},
		 {ADAU1979_REG_DC_HPF_CAL		,	0x00},
		 {ADAU1979_REG_BLOCK_POWER_SAI	,	0x33},
		 {ADAU1979_REG_MISC_CONTROL		,	0x00}
};

/*=============  L O C A L    F U N C T I O N    P R O T O T Y P E S =============*/
static int ADAU_1962_Pllinit(void);
static int ADAU_1979_Pllinit(void);
void Switch_Configurator(void);
static int ADAU_1962_init(void);
static int ADAU_1979_init(void);
void SRU_Init(void);
int Sport_Init(void);
int Sport_Stop(void);
static int SPU_init(void);
static int Init_TWI(void);
static int Stop_TWI(void);
extern void ConfigSoftSwitches_ADC_DAC(void);
extern void ConfigSoftSwitches_ADAU_Reset(void);
extern void OggVorbisDecode(void);
void Process_Block(void);

volatile int DecRIndex=0;
volatile int DecWIndex=DECODE_SIZE;

#define IBUFF_SIZE 1000

volatile int DecRIndexBuff[IBUFF_SIZE];
volatile int DecWIndexBuff[IBUFF_SIZE];

/*==============  D E F I N E S  ===============*/

/*=============  D A T A  =============*/

/* Buffers */
int DACBuffer[2*DECODE_SIZE] __attribute__ ((section (".L1.data")));
ADI_CACHE_ALIGN int   int_SP0ABuffer1[SPORT_BUFF_SIZE] __attribute__ ((section (".L1.data")));
ADI_CACHE_ALIGN static int   int_SP0ABuffer2[SPORT_BUFF_SIZE] __attribute__ ((section (".L1.data")));

ADI_CACHE_ALIGN static int   int_SP0ABuffer4[SPORT_BUFF_SIZE] __attribute__ ((section (".L1.data")));
ADI_CACHE_ALIGN static int   int_SP0ABuffer5[SPORT_BUFF_SIZE] __attribute__ ((section (".L1.data")));

/* Destination Sport PDMA list */
ADI_PDMA_DESC_LIST iDESC_LIST_1_SP4A; __attribute__ ((section (".L1.data")))
ADI_PDMA_DESC_LIST iDESC_LIST_2_SP4A __attribute__ ((section (".L1.data")));

/* Source Sport PDMA list */
ADI_PDMA_DESC_LIST iSRC_LIST_1_SP4B __attribute__ ((section (".L1.data")));
ADI_PDMA_DESC_LIST iSRC_LIST_2_SP4B __attribute__ ((section (".L1.data")));


/* Count to track the number of callBacks for SPORT transfer */
volatile uint8_t CallbackCount = 0;

volatile uint32_t TestCallbackCount = 0;

/* Memory required for SPORT */
static uint8_t SPORTMemory4A[ADI_SPORT_MEMORY_SIZE] __attribute__ ((section (".L1.data")));
static uint8_t SPORTMemory4B[ADI_SPORT_MEMORY_SIZE] __attribute__ ((section (".L1.data")));

/* SPORT Handle */
static ADI_SPORT_HANDLE hSPORTDev4ATx;//TX
static ADI_SPORT_HANDLE hSPORTDev4BRx;//RX

/* Memory required for TWI */
uint8_t TwideviceMemory[ADI_TWI_MEMORY_SIZE];

/* TWI driver handle */
static ADI_TWI_HANDLE hTwiDevice;

/* SPU handle */
static ADI_SPU_HANDLE      ghSpu;

/* Memory required for the SPU operation */
uint8_t             SpuMemory[ADI_SPU_MEMORY_SIZE];


/* Dev buffer for configuring ADC-DAC through TWI*/
static uint8_t devBuffer[BUFFER_SIZE];

/* Flag to optionally bypass processing */
volatile bool BypassProcessing = false;

/* Write pointer for circular buffer access for FIR accelerator processing */
volatile unsigned int wp = WP_START;

/* Flag for synchronization between core and SPORT DMA */
volatile bool  BlockReady = false;

static void SPORTCallback(
    void        *pAppHandle,
    uint32_t     nEvent,
    void        *pArg
)
{
	int i;

    /* CASEOF (event type) */
    switch (nEvent)
    {
        /* CASE (buffer processed) */
        case ADI_SPORT_EVENT_RX_BUFFER_PROCESSED:

#if 1

        	CallbackCount +=1;
        	BlockReady = true;
#else

#if 1
        		TestCallbackCount +=1;
        		CallbackCount +=1;
        		if(CallbackCount==1)
        		{
        		   for(i=0;i<SPORT_BUFF_SIZE;i++)
        		   {
        			   int_SP0ABuffer1[i]=DACBuffer[DecRIndex];/*Copy ADC buffer to DAC buffer */
        			   DecRIndex=(DecRIndex+1)%(2*DECODE_SIZE);
        		   }
        		}

        		if(CallbackCount==2)
				{
				   for(i=0;i<SPORT_BUFF_SIZE;i++)
				   {
					   int_SP0ABuffer2[i]=DACBuffer[DecRIndex];/*Copy ADC buffer to DAC buffer */
					   DecRIndex=(DecRIndex+1)%(2*DECODE_SIZE);
				   }
				   CallbackCount=0;
				}
#else
        		DecRIndex+=SPORT_BUFF_SIZE;
        		if(DecRIndex==2*DECODE_SIZE)
        		{
        			DecRIndex=0;
        		}
#endif
#endif
        		break;
        default:
        	 break;
    }



    /* return */
}

/*
 * Prepares SPU configuration.
 *
 * Parameters
 *  None
 *
 * Returns
 *  None
 *
 */
static int SPU_init(void)
{
    if(adi_spu_Init(0, SpuMemory, NULL, NULL, &ghSpu) != ADI_SPU_SUCCESS)
    {
    	REPORT_ERROR("Failed to initialize SPU service\n");
		return FAILED;
    }

    /* Make SPORT 0A to generate secure transactions */
    if(adi_spu_EnableMasterSecure(ghSpu, SPORT_4A_SPU, true) != ADI_SPU_SUCCESS)
    {
    	REPORT_ERROR("Failed to enable Master secure for SPORT0A\n");
		return FAILED;
    }

    /* Make SPORT 0B to generate secure transactions */
    if(adi_spu_EnableMasterSecure(ghSpu, SPORT_4B_SPU, true) != ADI_SPU_SUCCESS)
    {
    	REPORT_ERROR("Failed to enable Master secure for SPORT0B\n");
		return FAILED;
    }

    /* Make SPORT 0A to generate secure transactions */
	if(adi_spu_EnableMasterSecure(ghSpu, UART0_TxDMA_SPU_PID, true) != ADI_SPU_SUCCESS)
	{
		REPORT_ERROR("Failed to enable Master secure for UART0 TX\n");
		return FAILED;
	}

	/* Make SPORT 0A to generate secure transactions */
	if(adi_spu_EnableMasterSecure(ghSpu, UART0_RxDMA_SPU_PID, true) != ADI_SPU_SUCCESS)
	{
		REPORT_ERROR("Failed to enable Master secure for UART0 RX\n");
		return FAILED;
	}


    return SUCCESS;
}


void Switch_Configurator()
{
	int delay11=0xffff;

	SoftConfig_EV_21835_SOM();

	/* Software Switch Configuration for Enabling ADC-DAC */
	ConfigSoftSwitches_ADC_DAC();

	while(delay11--)
	{
		asm("nop;");
	}

	/* Software Switch Configuration for Re-Setting ADC-DAC  */
	ConfigSoftSwitches_ADAU_Reset();


	/* wait for Codec to up */
	delay11=0xffff;
	while(delay11--)
	{
		asm("nop;");
	}
}


/*
 * Prepares SRU configuration.
 *
 * Parameters
 *  None
 *
 * Returns
 *  None
 *
 */
void SRU_Init(void)
{
	*pREG_PADS0_DAI0_IE=0xfffff;
	*pREG_PADS0_DAI1_IE=0xfffff;

    SRU2(LOW,DAI1_PBEN05_I);

    SRU2(LOW,DAI1_PBEN04_I);

    SRU2(DAI1_PB05_O,DAI1_PB12_I);  /*DAC clock to ADC */
    SRU2(HIGH,DAI1_PBEN12_I);

    SRU2(DAI1_PB04_O,DAI1_PB20_I);  /*DAC FS to ADC */
    SRU2(HIGH,DAI1_PBEN20_I);

    SRU2(DAI1_PB06_O,SPT4_BD0_I);
    SRU2(LOW,DAI1_PBEN06_I);

    SRU2(HIGH,DAI1_PBEN01_I);

#ifdef ADD_ASRC

		SRU2(DAI1_PB05_O, SRC4_CLK_OP_I);
		SRU2(DAI1_PB04_O, SRC4_FS_OP_I);
		SRU2(SRC4_DAT_OP_O, DAI1_PB01_I);

	#ifdef USE_FPLL
		SRU2(HIGH, DAI1_PBEN20_I);
		SRU2(FRACPLL_CLK_O, DAI1_PB20_I);
		SRU2(DAI1_PB20_O, PCG0_EXTCLKA_I);

		SRU2(PCG0_CLKA_O, SRC4_CLK_IP_I);
		SRU2(PCG0_FSA_O, SRC4_FS_IP_I);
		SRU2(SPT4_AD0_O, SRC4_DAT_IP_I);

		SRU2(PCG0_CLKA_O,SPT4_ACLK_I); /*DAC clock to SPORT 4A*/
		SRU2(PCG0_FSA_O,SPT4_AFS_I);  /*DAC FS to SPORT 4A*/

		SRU2(PCG0_CLKA_O,SPT4_BCLK_I); /*DAC clock to SPORT 4A*/
		SRU2(PCG0_FSA_O,SPT4_BFS_I);  /*DAC FS to SPORT 4A*/

		*pREG_PCG0_CTLA0 = BITM_PCG_CTLA0_CLKEN|PCG_FSDIV|BITM_PCG_CTLA0_FSEN;
		//*pREG_PCG0_CTLA1 = PCG_CLKDIV|BITM_PCG_CTLA1_CLKSRC|BITM_PCG_CTLA1_FSSRC|((PCG_CLKDIV/2)<<BITP_PCG_CTLA1_FSPHASELO);
		*pREG_PCG0_CTLA1 = PCG_CLKDIV|BITM_PCG_CTLA1_CLKSRC|BITM_PCG_CTLA1_FSSRC|((PCG_CLKDIV/2)<<BITP_PCG_CTLA1_FSPHASELO);
		//*pREG_PCG0_SYNC1 = (1<<10)|(1<<11);

	#else

	#if 1
		SRU2(SPT4_ACLK_O, SRC4_CLK_IP_I);
		SRU2(SPT4_AFS_O, SRC4_FS_IP_I);
		SRU2(SPT4_AD0_O, SRC4_DAT_IP_I);

		SRU2(SPT4_ACLK_O,SPT4_BCLK_I); /*DAC clock to SPORT 4B*/
		SRU2(SPT4_AFS_O,SPT4_BFS_I);  /*DAC FS to SPORT 4B*/

	#else
		SRU2(DAI1_PB05_O, SRC4_CLK_IP_I);
		SRU2(DAI1_PB04_O, SRC4_FS_IP_I);
		SRU2(SPT4_AD0_O, SRC4_DAT_IP_I);

		SRU2(DAI1_PB05_O,SPT4_ACLK_I); /*DAC clock to SPORT 4A*/
		SRU2(DAI1_PB04_O,SPT4_AFS_I);  /*DAC FS to SPORT 4A*/

		SRU2(DAI1_PB05_O,SPT4_BCLK_I); /*DAC clock to SPORT 4A*/
		SRU2(DAI1_PB04_O,SPT4_BFS_I);  /*DAC FS to SPORT 4A*/
	#endif

#endif

#else
    SRU2(SPT4_AD0_O,DAI1_PB01_I); /* SPORT 4A to DAC*/

    SRU2(DAI1_PB05_O,SPT4_BCLK_I); /*DAC clock to SPORT 4B*/
    SRU2(DAI1_PB04_O,SPT4_BFS_I);  /*DAC FS to SPORT 4B*/

    SRU2(DAI1_PB05_O,SPT4_ACLK_I); /*DAC clock to SPORT 4A*/
    SRU2(DAI1_PB04_O,SPT4_AFS_I);  /*DAC FS to SPORT 4A*/
#endif


}

static void PrepareDescriptors (void)
{
#if defined(__ADSPCORTEXA55__)
	iDESC_LIST_1_SP4A.pStartAddr	= (uint32_t)(uintptr_t)(int *)int_SP0ABuffer1;
	iDESC_LIST_1_SP4A.pNxtDscp		= (uint32_t)(uintptr_t)&iDESC_LIST_2_SP4A;
#else
	iDESC_LIST_1_SP4A.pStartAddr	= (int *)int_SP0ABuffer1;
	iDESC_LIST_1_SP4A.pNxtDscp		= &iDESC_LIST_2_SP4A;
#endif
	iDESC_LIST_1_SP4A.Config		= ENUM_DMA_CFG_XCNT_INT ;
	iDESC_LIST_1_SP4A.XCount		= SPORT_BUFF_SIZE;
	iDESC_LIST_1_SP4A.XModify		= 4;
	iDESC_LIST_1_SP4A.YCount		= 0;
	iDESC_LIST_1_SP4A.YModify		= 0;

#if defined(__ADSPCORTEXA55__)
	iDESC_LIST_2_SP4A.pStartAddr	= (uint32_t)(uintptr_t)(int *)int_SP0ABuffer2;
	iDESC_LIST_2_SP4A.pNxtDscp		= (uint32_t)(uintptr_t)&iDESC_LIST_1_SP4A;
#else
	iDESC_LIST_2_SP4A.pStartAddr	= (int *)int_SP0ABuffer2;
	iDESC_LIST_2_SP4A.pNxtDscp		= &iDESC_LIST_1_SP4A;
#endif
	iDESC_LIST_2_SP4A.Config		= ENUM_DMA_CFG_XCNT_INT ;
	iDESC_LIST_2_SP4A.XCount		= SPORT_BUFF_SIZE;
	iDESC_LIST_2_SP4A.XModify		= 4;
	iDESC_LIST_2_SP4A.YCount		= 0;
	iDESC_LIST_2_SP4A.YModify		= 0;

#if defined(__ADSPCORTEXA55__)
	iSRC_LIST_1_SP4B.pStartAddr		= (uint32_t)(uintptr_t)(int *)int_SP0ABuffer4;
	iSRC_LIST_1_SP4B.pNxtDscp		= (uint32_t)(uintptr_t)&iSRC_LIST_2_SP4B;
#else
	iSRC_LIST_1_SP4B.pStartAddr		=(int *)int_SP0ABuffer4;
	iSRC_LIST_1_SP4B.pNxtDscp		= &iSRC_LIST_2_SP4B;
#endif
	iSRC_LIST_1_SP4B.Config			= ENUM_DMA_CFG_XCNT_INT ;
	iSRC_LIST_1_SP4B.XCount			= SPORT_BUFF_SIZE;
	iSRC_LIST_1_SP4B.XModify		= 4;
	iSRC_LIST_1_SP4B.YCount			= 0;
	iSRC_LIST_1_SP4B.YModify		= 0;

#if defined(__ADSPCORTEXA55__)
	iSRC_LIST_2_SP4B.pStartAddr		= (uint32_t)(uintptr_t)(int *)int_SP0ABuffer5;
	iSRC_LIST_2_SP4B.pNxtDscp		= (uint32_t)(uintptr_t)&iSRC_LIST_1_SP4B;
#else
	iSRC_LIST_2_SP4B.pStartAddr		=(int *)int_SP0ABuffer5;
	iSRC_LIST_2_SP4B.pNxtDscp		= &iSRC_LIST_1_SP4B;
#endif
	iSRC_LIST_2_SP4B.Config			= ENUM_DMA_CFG_XCNT_INT;
	iSRC_LIST_2_SP4B.XCount			= SPORT_BUFF_SIZE;
	iSRC_LIST_2_SP4B.XModify		= 4;
	iSRC_LIST_2_SP4B.YCount			= 0;
	iSRC_LIST_2_SP4B.YModify		= 0;

}

int Sport_Init(void)
{
    /* SPORT return code */
    ADI_SPORT_RESULT    eResult;

	/* Open the SPORT Device 4A */
	eResult = adi_sport_Open(SPORT_DEVICE_4A,ADI_HALF_SPORT_A,ADI_SPORT_DIR_TX, ADI_SPORT_I2S_MODE, SPORTMemory4A,ADI_SPORT_MEMORY_SIZE,&hSPORTDev4ATx);
	CHECK_RESULT(eResult);
	/* Open the SPORT Device 4B*/
	eResult = adi_sport_Open(SPORT_DEVICE_4B,ADI_HALF_SPORT_B,ADI_SPORT_DIR_RX, ADI_SPORT_I2S_MODE, SPORTMemory4B,ADI_SPORT_MEMORY_SIZE,&hSPORTDev4BRx);
	CHECK_RESULT(eResult);

/* Configure SPORT in internal clock and FS mode */
#ifdef ADD_ASRC

#ifndef USE_FPLL
#if 1
	eResult = adi_sport_ConfigClock(hSPORTDev4ATx, SPORT_CLK_DIV, true, false, false);
	CHECK_RESULT(eResult);

	eResult = adi_sport_ConfigFrameSync(hSPORTDev4ATx, SPORT_FS_DIV, true, true, true, true, false, true);
	CHECK_RESULT(eResult);
#endif
#endif

	*pREG_ASRC1_CTL01 = BITM_ASRC_CTL01_EN0|(1<<BITP_ASRC_CTL01_SMODEIN0)|(1<<BITP_ASRC_CTL01_SMODEOUT0);
#endif

	/* Register SPORT Callback function */
	eResult = adi_sport_RegisterCallback(hSPORTDev4BRx,SPORTCallback,NULL);
	CHECK_RESULT(eResult);

	/* Prepare descriptors */
	PrepareDescriptors();

	/* Submit the first buffer for Rx.  */
	eResult = adi_sport_DMATransfer(hSPORTDev4BRx,&iSRC_LIST_1_SP4B,(DMA_NUM_DESC),ADI_PDMA_DESCRIPTOR_LIST, ADI_SPORT_CHANNEL_PRIM);
	CHECK_RESULT(eResult);
	/* Submit the first buffer for Tx.  */
	eResult = adi_sport_DMATransfer(hSPORTDev4ATx,&iDESC_LIST_1_SP4A,(DMA_NUM_DESC),ADI_PDMA_DESCRIPTOR_LIST, ADI_SPORT_CHANNEL_PRIM);
	CHECK_RESULT(eResult);

	/*Enable the Sport Device 4B */
	eResult = adi_sport_Enable(hSPORTDev4BRx,true);
	CHECK_RESULT(eResult);
	/*Enable the Sport Device 4A */
	eResult = adi_sport_Enable(hSPORTDev4ATx,true);
	CHECK_RESULT(eResult);

	return eResult;

}


int Sport_Stop(void)
{
    /* SPORT return code */
    ADI_SPORT_RESULT    eResult;

    /*Stop the DMA transfer of  Sport Device 4B */
	eResult = adi_sport_StopDMATransfer(hSPORTDev4BRx);
	CHECK_RESULT(eResult);
	/*Stop the DMA transfer of  Sport Device 4A */
	eResult = adi_sport_StopDMATransfer(hSPORTDev4ATx);
	CHECK_RESULT(eResult);
	/*Close Sport Device 4B */
	eResult = adi_sport_Close(hSPORTDev4BRx);
	CHECK_RESULT(eResult);
	/*Close Sport Device 4A */
	eResult = adi_sport_Close(hSPORTDev4ATx);
	CHECK_RESULT(eResult);

	return eResult;
}

void Write_TWI_8bit_Reg(unsigned char Reg_ID, unsigned char Tx_Data)
{
	devBuffer[0] = Reg_ID;
	devBuffer[1] = Tx_Data;
	adi_twi_Write(hTwiDevice, devBuffer, 2u, false);
}

unsigned char Read_TWI_8bit_Reg(unsigned char Reg_ID)
{
	ADI_TWI_RESULT eResult;
	unsigned char Rx_Data;

	/* write register address */
	devBuffer[0] = Reg_ID;
	eResult = adi_twi_Write(hTwiDevice, devBuffer, 1u, true);
	if(eResult!=ADI_TWI_SUCCESS)
	{
		REPORT_ERROR("TWI write failed 0x%08X\n", eResult);
	}

	/* read register value */
	eResult = adi_twi_Read(hTwiDevice, &Rx_Data, 1u, false);
	if(eResult!=ADI_TWI_SUCCESS)
	{
		REPORT_ERROR("TWI Read failed 0x%08X\n", eResult);
	}

	return Rx_Data;
}

static int Init_TWI(void)
{

	ADI_TWI_RESULT eResult;

	eResult = adi_twi_Open(TWIDEVNUM, ADI_TWI_MASTER, &TwideviceMemory[0],
			   	ADI_TWI_MEMORY_SIZE, &hTwiDevice);
	if(eResult!=ADI_TWI_SUCCESS)
	{
		REPORT_ERROR("TWI Open failed 0x%08X\n", eResult);
	}

	eResult = adi_twi_SetPrescale(hTwiDevice, PRESCALEVALUE);
	if(eResult!=ADI_TWI_SUCCESS)
	{
		REPORT_ERROR("TWI Set Prescale failed 0x%08X\n", eResult);
	}

	eResult = adi_twi_SetBitRate(hTwiDevice, BITRATE);
	if(eResult!=ADI_TWI_SUCCESS)
	{
		REPORT_ERROR("TWI Set Bitrate failed 0x%08X\n", eResult);
	}


	eResult = adi_twi_SetDutyCycle(hTwiDevice, DUTYCYCLE);
	if(eResult!=ADI_TWI_SUCCESS)
	{
		REPORT_ERROR("TWI Set Duty cycle failed 0x%08X\n", eResult);
	}

	eResult = adi_twi_SetHardwareAddress(hTwiDevice, TARGETADDR);
	if(eResult!=ADI_TWI_SUCCESS)
	{
		REPORT_ERROR("TWI Set Hw address failed 0x%08X\n", eResult);
	}

	return eResult;
}

static int Stop_TWI(void)
{
	ADI_TWI_RESULT eResult;

	eResult = adi_twi_Close(hTwiDevice);
	CHECK_RESULT(eResult);

	return eResult;
}

/*****************************************************************************************************************************/

static int ADAU_1962_init(void)
{	int i;
	ADAU_1962_Pllinit();
	for(i=0;i<28;i++)
	{
		/* write value */
		Write_TWI_8bit_Reg(Config_array_DAC[i].Reg_Add,Config_array_DAC[i].Value);
		Config_read_DAC[i]=Read_TWI_8bit_Reg(Config_array_DAC[i].Reg_Add);
		if(Config_array_DAC[i].Value!= Config_read_DAC[i])
		{
			DEBUG_INFORMATION("\n Configuring ADAU_1962 failed");
			return FAILED;
		}
	}

	return SUCCESS;
}

static int ADAU_1962_Pllinit(void)
{
	int status,delay1=0xffff;

	ADI_TWI_RESULT eResult;

   	eResult = adi_twi_SetHardwareAddress(hTwiDevice, TARGETADDR_1962);
	if(eResult!=ADI_TWI_SUCCESS)
	{
		REPORT_ERROR("TWI Set Hw address failed 0x%08X\n", eResult);
	}

	Write_TWI_8bit_Reg(ADAU1962_PLL_CTL_CTRL0,0x01);
	while(delay1--)
	{
		asm("nop;");
	}

	Write_TWI_8bit_Reg(ADAU1962_PLL_CTL_CTRL0,0x05);

	delay1=0xffff;
	while(delay1--)
	{
		asm("nop;");
	}

	Write_TWI_8bit_Reg(ADAU1962_PLL_CTL_CTRL1,0x22);
	delay1=0xffff;
	while(delay1--)
	{
		asm("nop;");
	}
	status=Read_TWI_8bit_Reg(ADAU1962_PLL_CTL_CTRL1);
	while(!((status & 0x4)>>2)  )
	{
		status=Read_TWI_8bit_Reg(ADAU1962_PLL_CTL_CTRL1);
	}

	return eResult;
}




static int ADAU_1979_init(void)
{	int i;

	ADAU_1979_Pllinit();

	for(i=0;i<16;i++)
	{
		Write_TWI_8bit_Reg(Config_array_ADC[i].Reg_Add,Config_array_ADC[i].Value);
		Config_read_ADC[i]=Read_TWI_8bit_Reg(Config_array_ADC[i].Reg_Add);
		if(Config_array_ADC[i].Value!= Config_read_ADC[i])
		{
			DEBUG_INFORMATION("\n Configuring ADAU_1979 failed");
			return FAILED;
		}
	}

	return SUCCESS;
}

static int ADAU_1979_Pllinit(void)
{

	int status,delay1=0xffff;

	ADI_TWI_RESULT eResult;

   	eResult = adi_twi_SetHardwareAddress(hTwiDevice, TARGETADDR_1979);
	if(eResult!=ADI_TWI_SUCCESS)
	{
		REPORT_ERROR("TWI Set Hw address failed 0x%08X\n", eResult);
	}

   	Write_TWI_8bit_Reg(ADAU1979_REG_POWER,0x01);
	Write_TWI_8bit_Reg(ADAU1979_REG_PLL,0x03);
	status=Read_TWI_8bit_Reg(ADAU1979_REG_PLL);
	while(delay1--)
			{
				asm("nop;");
			}
	while(!((status & 0x80)>>7)  )
	{
		status=Read_TWI_8bit_Reg(ADAU1979_REG_PLL);
		asm("nop;");
	}

	return eResult;

}

/* FPLL device handle */
static ADI_FPLL_HANDLE hfpll;

/*
 * Main function
 */
int main(int argc, char *argv[])
{
	/**
	 * Initialize managed drivers and/or services that have been added to
	 * the project.
	 * @return zero on success
	 */

	uint32_t Result=0;

	adi_initComponents();

	//DEBUG_INFORMATION( "ADC/DAC Audio talk-through test I2S\n" );

    /* SPU initialization */
	if (Result==0u)
	{
		Result=SPU_init();
	}

    /* Switch Configuration */
	Switch_Configurator();

	/* SRU Configuration */
	SRU_Init();

#ifdef USE_FPLL
	/* Open FPLL device */
	adi_fpll_Open(0, &hfpll);

	/* Generate FPLL output and probe the output */
	adi_fpll_FPLLClockConfig(hfpll, ADI_FPLL_CLK_FREQ_22_5792M);
#endif

	/* TWI Initialization */
	if (Result==0u)
	{
		Result=Init_TWI();
	}

	/* ADAU1962 Initialization */
	if (Result==0u)
	{
		Result=ADAU_1962_init();
	}

	/* ADAU1979 Initialization */
	if (Result==0u)
	{
		Result=ADAU_1979_init();
	}

	/* One time FIR/IIR accelerator initialization */
	Init_FIR_IIR(ACCUsageModel);

	/* Initialize UART debug port */
	UartInit();

	/* SPORT Initialization */
	if (Result==0u)
	{
		Result=Sport_Init();
	}

	/* Close TWI */
	if (Result==0u)
	{
		Result=Stop_TWI();
	}


	/* Ogg-Vorbis Decoder  */
	OggVorbisDecode();

#if (ADI_CONFIG_CONTINUOUS_AUDIO==0)

	while(1)
	{
		if(TestCallbackCount == 8000)
		{
			int delay=0xffffff;
			while(delay--)
			{
				asm("nop;");
			}

			if (Result==0u)
			{
				Result=Sport_Stop();
			}

			if(Result==0u)
			{
				DEBUG_INFORMATION("All done\n");
			}
			else
			{
				DEBUG_INFORMATION("Example failed.\n");
			}

			break;
		}
	}
#elif (ADI_CONFIG_CONTINUOUS_AUDIO==1)
	while(1);
#endif
}

/* Convert fixed point data to floating point with 2^-31 scaling */
void Block_Fixed_To_Float( int * Fixed_In, float * Float_Out_L, float * Float_Out_R, int size )
{
	int i;

	for (i=0;i<size;i++)
	{
		Float_Out_L[i]  = ((float) (Fixed_In[2*i]<<8))   * (1.0/2147483648.0);
		Float_Out_R[i]  = ((float) (Fixed_In[2*i+1]<<8)) * (1.0/2147483648.0);
	}
}

/* Convert floating point data to fixed point with 2^-31 scaling */
void Block_Float_To_Fixed( int * Fixed_Out, float * Float_In_L, float * Float_In_R, int size  )
{

	int i;
	for (i=0;i<size;i++)
	{
		Fixed_Out[2*i]   = ((int) (2147483648.0*Float_In_L[i]))>>8;
		Fixed_Out[2*i+1] = ((int) (2147483648.0*Float_In_R[i]))>>8;
	}
}

/* Function to process a block of data, total 12 channels are processed but only the first two channels use valid audio data */
void Process_Block(void)
{
	int i;

	/* Ping buffer */
	if(CallbackCount==1)
	{
		if(BypassProcessing==false)
		{
			 /* Prepare input buffers for accelerator processing for direct replacement and data pipelining usage models */
			/* Accelerator needs input buffer to hold at least BLOCK_SIZE+FIR_TAPS-1 words for delay line preload, thus we need to maintain circular buffering */
			if((ACCUsageModel==DIRECT_REPLACEMENT)||(ACCUsageModel==DATA_PIPELINING))
			{
					Block_Fixed_To_Float( (int*)&DACBuffer[DecRIndex], &Input[0][wp], &Input[1][wp], BLOCK_SIZE );
					DecRIndex=(DecRIndex+SPORT_BUFF_SIZE)%(2*DECODE_SIZE);
					wp+=BLOCK_SIZE;
					wp=wp%(CIRCULAR_BUFFER_LENGTH);
			}
			/* Prepare input buffers for core processing for core only usage model */
			else if(ACCUsageModel==CORE_ONLY)
			{
					Block_Fixed_To_Float( (int*)&DACBuffer[DecRIndex], Input[0], Input[1], BLOCK_SIZE );
					DecRIndex=(DecRIndex+SPORT_BUFF_SIZE)%(2*DECODE_SIZE);
			}
			/* Prepare input buffers for core and accelerator processing for split task usage model */
			else if(ACCUsageModel==SPLIT_TASK)
			{
				/* Accelerator needs input buffer to hold at least BLOCK_SIZE+FIR_TAPS-1 words for delay line preload, thus we need to maintain circular buffering */
				Block_Fixed_To_Float( (int*)&DACBuffer[DecRIndex], &Input[0][wp], &Input[1][wp], BLOCK_SIZE );
				wp+=BLOCK_SIZE;
				wp=wp%(CIRCULAR_BUFFER_LENGTH);

				Block_Fixed_To_Float( (int*)&DACBuffer[DecRIndex], Input[10], Input[11], BLOCK_SIZE );
				DecRIndex=(DecRIndex+SPORT_BUFF_SIZE)%(2*DECODE_SIZE);
			}

		   /* Call FIR/IIR processing function */
		   Process_FIR_IIR(ACCUsageModel);

		   /* Use FIR/IIR processing output for output buffer preparation */
		   if(ACCUsageModel==SPLIT_TASK)
		   {
			   Block_Float_To_Fixed( (int*)&int_SP0ABuffer1, Output[0], Output[11], BLOCK_SIZE  );
		   }
		   else
		   {
			   Block_Float_To_Fixed( (int*)&int_SP0ABuffer1, Output[0], Output[1], BLOCK_SIZE  );
		   }
		}
		else
		{
			for(i=0;i<SPORT_BUFF_SIZE;i++)
		    {
			   int_SP0ABuffer1[i]=DACBuffer[DecRIndex]; /*Copy ADC buffer to DAC buffer */
			   DecRIndex=(DecRIndex+1)%(2*DECODE_SIZE);
		    }

			RESET_MIPS
		}
	}

	/* Pong buffer */
	if(CallbackCount==2)
	{
		if(BypassProcessing==false)
		{
			 /* Prepare input buffers for accelerator processing for direct replacement and data pipelining usage models */
			if((ACCUsageModel==DIRECT_REPLACEMENT)||(ACCUsageModel==DATA_PIPELINING))
			{
					Block_Fixed_To_Float( (int*)&DACBuffer[DecRIndex], &Input[0][wp], &Input[1][wp], BLOCK_SIZE );
					DecRIndex=(DecRIndex+SPORT_BUFF_SIZE)%(2*DECODE_SIZE);
					wp+=BLOCK_SIZE;
					wp=wp%(CIRCULAR_BUFFER_LENGTH);
			   /* Core Processing */
			}
			/* Prepare input buffers for core processing for core only usage model */
			else if(ACCUsageModel==CORE_ONLY)
			{
					Block_Fixed_To_Float( (int*)&DACBuffer[DecRIndex], Input[0], Input[1], BLOCK_SIZE );
					DecRIndex=(DecRIndex+SPORT_BUFF_SIZE)%(2*DECODE_SIZE);
			}
			/* Prepare input buffers for core and accelerator processing for split task usage model */
			else if(ACCUsageModel==SPLIT_TASK)
			{
				/* Accelerator needs input buffer to hold at least BLOCK_SIZE+FIR_TAPS-1 words for delay line preload, thus we need to maintain circular buffering */
				Block_Fixed_To_Float( (int*)&DACBuffer[DecRIndex], &Input[0][wp], &Input[1][wp], BLOCK_SIZE );
				wp+=BLOCK_SIZE;
				wp=wp%(CIRCULAR_BUFFER_LENGTH);

				Block_Fixed_To_Float( (int*)&DACBuffer[DecRIndex], Input[10], Input[11], BLOCK_SIZE );
				DecRIndex=(DecRIndex+SPORT_BUFF_SIZE)%(2*DECODE_SIZE);
			}

			 /* Call FIR/IIR processing function */
			Process_FIR_IIR(ACCUsageModel);

			/* Use FIR/IIR processing output for output buffer preparation */
		   if(ACCUsageModel==SPLIT_TASK)
		   {
			   Block_Float_To_Fixed( (int*)&int_SP0ABuffer2, Output[0], Output[11], BLOCK_SIZE  );
		   }
		   else
		   {
			   Block_Float_To_Fixed( (int*)&int_SP0ABuffer2, Output[0], Output[1], BLOCK_SIZE  );
		   }
		}
		else
		{
			for(i=0;i<SPORT_BUFF_SIZE;i++)
		    {
				   int_SP0ABuffer2[i]=DACBuffer[DecRIndex]; /*Copy ADC buffer to DAC buffer */
				   DecRIndex=(DecRIndex+1)%(2*DECODE_SIZE);
		    }

			RESET_MIPS
		}

		CallbackCount=0;
	}
}

