/* This file contains the code to do FIR/IIR processing */

#include "Processing.h"

volatile int cache_cycles=0;
volatile int cc=0;

//#define USE_L2
//#define USE_L3

#if defined(USE_L2)

/* Core Input Buffer */
float Input[NCH][CIRCULAR_BUFFER_LENGTH] __attribute__ ((section (".L2.data")));

/* IIR Output Buffers */
float IIROutput[NCH][BLOCK_SIZE*IIR_BANDS] __attribute__ ((section (".L2.data")));

/* FIR Output buffers */
float FIROutput[NCH][BLOCK_SIZE] __attribute__ ((section (".L2.data")));

/* Final Output Buffer */
float Output[NCH][BLOCK_SIZE] __attribute__ ((section (".L2.data")));


/* FIR Core/ACC coefficients */
float __attribute__ ((section (".L2.data"))) FIRCoeffs[FIR_TAPS] =
{
		#include "fir_coeffs.dat"
};

/* FIR Core/ACC coefficients */
float __attribute__ ((section (".L2.data"))) FIRCoeffsCore[FIR_TAPS] =
{
		#include "fir_coeffs_core.dat"
};

/* IIR Coefficients for core processing*/
float __attribute__ ((section (".L2.data"))) IIRCoeffs[NCH][IIR_BANDS*NSECTIONS*5] =
{
		#include "iirc_coeffs.dat"
};

/* IIR delay line for core processing */
float __attribute__ ((section (".L2.data"))) iir_state[NCH][2*NSECTIONS*IIR_BANDS];

/* IIR coefficients for ACC processing */
float __attribute__ ((section (".L2.data"))) IIRCoeffsACC[NCH][IIR_BANDS*NSECTIONS*7] =
{
		#include "iira_coeffs.dat"
};
unsigned char iirobj[NCH][IIR_BANDS][592]  __attribute__ ((section (".L2.data"))); // = malloc (s_iirf_df2_alloc (NSECTIONS, 1)); // __attribute__ ((section (".L1.data")));

fir_statef firobj[NCH] __attribute__ ((section (".L2.data")));


#elif defined(USE_L3)
/* Core Input Buffer */
float Input[NCH][CIRCULAR_BUFFER_LENGTH] __attribute__ ((section (".L3.data")));

/* IIR Output Buffers */
float IIROutput[NCH][BLOCK_SIZE*IIR_BANDS] __attribute__ ((section (".L3.data")));

/* FIR Output buffers */
float FIROutput[NCH][BLOCK_SIZE] __attribute__ ((section (".L3.data")));

/* Final Output Buffer */
float Output[NCH][BLOCK_SIZE] __attribute__ ((section (".L3.data")));


/* FIR Core/ACC coefficients */
float __attribute__ ((section (".L3.data"))) FIRCoeffs[FIR_TAPS] =
{
		#include "fir_coeffs.dat"
};

/* FIR Core/ACC coefficients */
float __attribute__ ((section (".L3.data"))) FIRCoeffsCore[FIR_TAPS] =
{
		#include "fir_coeffs_core.dat"
};

/* IIR Coefficients for core processing*/
float __attribute__ ((section (".L3.data"))) IIRCoeffs[NCH][IIR_BANDS*NSECTIONS*5] =
{
		#include "iirc_coeffs.dat"
};

/* IIR delay line for core processing */
float __attribute__ ((section (".L3.data"))) iir_state[NCH][2*NSECTIONS*IIR_BANDS];

/* IIR coefficients for ACC processing */
float __attribute__ ((section (".L3.data"))) IIRCoeffsACC[NCH][IIR_BANDS*NSECTIONS*7] =
{
		#include "iira_coeffs.dat"
};
unsigned char iirobj[NCH][IIR_BANDS][592]  __attribute__ ((section (".L3.data"))); // = malloc (s_iirf_df2_alloc (NSECTIONS, 1)); // __attribute__ ((section (".L1.data")));

fir_statef firobj[NCH] __attribute__ ((section (".L3.data")));

#else
/* Core Input Buffer */
float Input[NCH][CIRCULAR_BUFFER_LENGTH] __attribute__ ((section (".L1.data")));

/* IIR Output Buffers */
float IIROutput[NCH][BLOCK_SIZE*IIR_BANDS] __attribute__ ((section (".L1.data")));

/* FIR Output buffers */
float FIROutput[NCH][BLOCK_SIZE] __attribute__ ((section (".L1.data")));

/* Final Output Buffer */
float Output[NCH][BLOCK_SIZE] __attribute__ ((section (".L3.data")));

/* FIR Core/ACC coefficients */
float __attribute__ ((section (".L1.data"))) FIRCoeffs[FIR_TAPS] =
{
		#include "fir_coeffs.dat"
};

/* FIR Core/ACC coefficients */
float __attribute__ ((section (".L1.data"))) FIRCoeffsCore[FIR_TAPS] =
{
		#include "fir_coeffs_core.dat"
};

/* IIR Coefficients for core processing*/
float __attribute__ ((section (".L1.data"))) IIRCoeffs[NCH][IIR_BANDS*NSECTIONS*5] =
{
		#include "iirc_coeffs.dat"
};

/* IIR delay line for core processing */
float __attribute__ ((section (".L1.data"))) iir_state[NCH][2*NSECTIONS*IIR_BANDS];

/* IIR coefficients for ACC processing */
float __attribute__ ((section (".L1.data"))) IIRCoeffsACC[NCH][IIR_BANDS*NSECTIONS*7] =
{
		#include "iira_coeffs.dat"
};
unsigned char iirobj[NCH][IIR_BANDS][592]  __attribute__ ((section (".L1.data"))); // = malloc (s_iirf_df2_alloc (NSECTIONS, 1)); // __attribute__ ((section (".L1.data")));

fir_statef firobj[NCH] __attribute__ ((section (".L1.data")));

#endif
/* FIR task memory for ACC*/
#pragma align 32
uint8_t FirTaskMemory[FIRA_INSTANCES][FIR_MEM_SIZE(NCH/2)] __attribute__ ((section (".L1.data")));

/* IIR task memory for ACC */
#pragma align 32
uint8_t IirTaskMemory[NCH][IIR_MEM_SIZE(IIR_BANDS)] __attribute__ ((section (".L1.data")));

/* FIR device handles */
ADI_FIR_DEV_HANDLE hFir[FIRA_INSTANCES];

/* FIR task handle */
ADI_FIR_TASK_HANDLE hFirTask[NCH_ACC] __attribute__ ((section (".L1.data")));

/* Variable to track FIR task done */
volatile uint32_t FIRTaskDone = 0;

/* FIR channel information */
ADI_FIR_CHANNEL_INFO FirTaskChannels[NCH];

/* FIR task done callback */
void FIRTaskDoneCallback( void * pCBParam, ADI_FIR_EVENT Event, void * pArg);

/* IIR device handle */
ADI_IIR_DEV_HANDLE hIir[IIRA_INSTANCES] __attribute__ ((section (".L1.data")));

/* IIR task handle */
ADI_IIR_TASK_HANDLE hIirTasks[NCH] __attribute__ ((section (".L1.data")));

/* IIR channel information */
ADI_IIR_CHANNEL_INFO IirTaskChannels[NCH][IIR_BANDS] __attribute__ ((section (".L1.data")));

/* Variable to track FIR tasks done */
volatile uint32_t IIRTaskDoneCount = 0;

/* IIR task done callback */
void IIRTaskDoneCallback( void * pCBParam, ADI_IIR_EVENT Event, void * pArg);

/* Function to initialize FIR/IIR channel information for FIR/IIR driver */
void InitializeFIRTaskChannels(int nch);
void InitializeIIRTaskChannels(int nch);

/* IIR gains */
float IIRGains[IIR_BANDS] = {1.0, 1.0, 1.0, 1.0, 1.0};

float IIRGains1[NCH][IIR_BANDS] = {
				#include "gains.dat"
};

float IIRGainsFinal[NCH][IIR_BANDS];

/* This flag is used to perform one time initialization while calling INIT_FIR_IIR function, it is initially zero and set to 1 after calling the
 * INIT_FIR_IIR function once.
 */
volatile int init_flag=0;

/* Functions required to do SPORT init and SPORT stop needed for framework reinitialization */
extern int Sport_Init(void);

extern int Sport_Stop(void);


/* One time initialization for FIR/IIR processing */
void Init_FIR_IIR(ACC_USAGE_MODEL ACCUsageModel)
{
	int i,j,k;

	/* FIR result */
	ADI_FIR_RESULT eFirResult = ADI_FIR_RESULT_SUCCESS;

	/* IIR result */
	ADI_IIR_RESULT eIirResult = ADI_IIR_RESULT_SUCCESS;

	ADI_FIR_TASK_STATE FIRTS;

	ADI_IIR_TASK_STATE IIRTS;

	/* Reset the input buffer write pointer for accelerator processing */
	wp = WP_START;

	/* One time initialization */
	if(init_flag==0)
	{
		/* Initialize FIR task channels */
		InitializeFIRTaskChannels(NCH);

		/* Initialize IIR task channels */
		InitializeIIRTaskChannels(NCH);

		/* Open all FIR device instances */
		for(i=0; i<FIRA_INSTANCES; i++)
		{
			eFirResult  = adi_fir_Open(i,&hFir[i]);
			CHECK_FIR_RESULT(eFirResult);
		}

		/* Register Callback for all instances*/
		eFirResult = adi_fir_RegisterCallback(hFir[0],FIRTaskDoneCallback,0);
		CHECK_FIR_RESULT(eFirResult)

		eFirResult = adi_fir_RegisterCallback(hFir[1],FIRTaskDoneCallback,0);
		CHECK_FIR_RESULT(eFirResult)

		/* Create FIRA Tasks */
		eFirResult = adi_fir_CreateTask(hFir[0], /* Device Handle*/
				FirTaskChannels,              /* Pointer to Channel List */
				NCH/2,					  	  /* Number of Channels  */
				&FirTaskMemory[0],               /* Pointer to Memory */
				FIR_MEM_SIZE(NCH/2),           	/*Memory Size */
				&hFirTask[0]);               /* Address to store the handle */
		CHECK_FIR_RESULT(eFirResult)

		eFirResult = adi_fir_CreateTask(hFir[1], /* Device Handle*/
				&FirTaskChannels[NCH/2],       /* Pointer to Channel List */
				NCH/2,					  	  /* Number of Channels  */
				&FirTaskMemory[1],               /* Pointer to Memory */
				FIR_MEM_SIZE(NCH/2),           	/*Memory Size */
				&hFirTask[1]);               /* Address to store the handle */
		CHECK_FIR_RESULT(eFirResult)

		/* Open all IIR Device instances  */
		for(i=0; i<IIRA_INSTANCES; i++)
		{
			eIirResult  = adi_iir_Open(i,&hIir[i]);
			CHECK_IIR_RESULT(eIirResult);
		}

		/* Register the Callback */
		eIirResult = adi_iir_RegisterCallback(hIir[0],IIRTaskDoneCallback,0);
		CHECK_IIR_RESULT(eIirResult);

		eIirResult = adi_iir_RegisterCallback(hIir[1],IIRTaskDoneCallback,0);
		CHECK_IIR_RESULT(eIirResult);

		eIirResult = adi_iir_RegisterCallback(hIir[2],IIRTaskDoneCallback,0);
		CHECK_IIR_RESULT(eIirResult);

		eIirResult = adi_iir_RegisterCallback(hIir[3],IIRTaskDoneCallback,0);
		CHECK_IIR_RESULT(eIirResult);

#if 0
		eIirResult = adi_iir_RegisterCallback(hIir[4],IIRTaskDoneCallback,0);
		CHECK_IIR_RESULT(eIirResult);

		eIirResult = adi_iir_RegisterCallback(hIir[5],IIRTaskDoneCallback,0);
		CHECK_IIR_RESULT(eIirResult);

		eIirResult = adi_iir_RegisterCallback(hIir[6],IIRTaskDoneCallback,0);
		CHECK_IIR_RESULT(eIirResult);

		eIirResult = adi_iir_RegisterCallback(hIir[7],IIRTaskDoneCallback,0);
		CHECK_IIR_RESULT(eIirResult);
#endif

		for(i=0;i<NCH;i++)
		{
			if(i<4)
			{
				eIirResult = adi_iir_CreateTask(hIir[i], 			  /* Device Handle */
										IirTaskChannels[i],           /* Pointer to Channel List */
										5, 							  /* Number of Channels */
										&IirTaskMemory[i],            /* Pointer to Task Memory */
										IIR_MEM_SIZE(IIR_BANDS),      /* Memory Size */
										&hIirTasks[i]);               /* Address to store handle */
				CHECK_IIR_RESULT(eIirResult);
			}
			if(i>=4&&i<8)
			{
				eIirResult = adi_iir_CreateTask(hIir[i-4], 				/* Device Handle */
										IirTaskChannels[i],           /* Pointer to Channel List */
										5, 							  /* Number of Channels */
										&IirTaskMemory[i],            /* Pointer to Task Memory */
										IIR_MEM_SIZE(IIR_BANDS),      /* Memory Size */
										&hIirTasks[i]);               /* Address to store handle */
				CHECK_IIR_RESULT(eIirResult);
			}
			if(i>=8&&i<12)
			{
				eIirResult = adi_iir_CreateTask(hIir[i-8], 				/* Device Handle */
										IirTaskChannels[i],           /* Pointer to Channel List */
										5, 							  /* Number of Channels */
										&IirTaskMemory[i],            /* Pointer to Task Memory */
										IIR_MEM_SIZE(IIR_BANDS),      /* Memory Size */
										&hIirTasks[i]);               /* Address to store handle */
				CHECK_IIR_RESULT(eIirResult);
			}

			for(j=0;j<IIR_BANDS;j++)
			{
				IIRGainsFinal[i][j]=IIRGains[j]*IIRGains1[i][j];
			}
		}
	}
	else
	{
		//Sport_Stop(); /* Stop SPORTs */
	}

	/* Initialize FIR/IIR state arrays to zero for Core Only usage model */
	if(ACCUsageModel==CORE_ONLY)
	{
		for(i=0;i<NCH;i++)
		{
			for(j=0;j<FIR_TAPS+1;j++)
				Input[i][j+BLOCK_SIZE]=0;

			for(j=0;j<2*NSECTIONS*IIR_BANDS;j++)
			{
				iir_state[i][j]=0;

			}

			//printf("\n\nsize=%d", s_iirf_df2_alloc (NSECTIONS, 1) );
			//printf("\n\nsize=%d", s_iirf_df2_alloc (NSECTIONS, 1) );

			for(j=0;j<IIR_BANDS;j++)
			{
				s_iirf_df2_init ((s_iirf_df2_t *)(&iirobj[i][j]), NSECTIONS, 1, &IIRCoeffs[i][j*NSECTIONS*5]);
			}


			fir_init(&firobj[i], FIRCoeffsCore, &Input[i], FIR_TAPS, 1);
		}


	}
	else
	{
		/* Delete the previous FIR/IIR tasks */
		if(hFirTask!=NULL)
		{
			/* Wait till previous FIR task is over */
			do
			{
				adi_fir_GetFirTaskStatus(hFirTask[0], &FIRTS);
			}
			while(FIRTS==ADI_IIR_TASK_STATE_RUNNING);

			do
			{
				adi_fir_GetFirTaskStatus(hFirTask[1], &FIRTS);
			}
			while(FIRTS==ADI_IIR_TASK_STATE_RUNNING);

			/* Delete FIR task */
			adi_fir_DeleteTask(hFirTask[0]);
			CHECK_FIR_RESULT(eFirResult);

			adi_fir_DeleteTask(hFirTask[1]);
			CHECK_FIR_RESULT(eFirResult);

			for(i=0;i<NCH;i++)
			{
				if(hIirTasks[i]!=NULL)
				{
					/* Wait till previous IIR task is over */
					do
					{
						adi_iir_GetIirTaskStatus(hIirTasks[i], &IIRTS);
					}
					while(IIRTS==ADI_IIR_TASK_STATE_RUNNING);

					/* Delete IIR task */
					eIirResult = adi_iir_DeleteTask(hIirTasks[i]);
				}
				CHECK_FIR_RESULT(eIirResult);
			}
		}

		/* Reset FIR task memory */
		memset(&FirTaskMemory[0], 0, FIR_MEM_SIZE(NCH/2));
		memset(&FirTaskMemory[1], 0, FIR_MEM_SIZE(NCH/2));

		for(i=0;i<NCH;i++)
		{
			/* Reset IIR task memory */
			memset(&IirTaskMemory[i], 0, IIR_MEM_SIZE(IIR_BANDS));

			/* Reset IIR state */
			for(j=0;j<IIR_BANDS;j++)
			{
				for(k=0;k<NSECTIONS;k++)
				{
					int temp = j*NSECTIONS*7+k*7;

					IIRCoeffsACC[i][temp+5]=0.0;
					IIRCoeffsACC[i][temp+6]=0.0;
				}
			}
		}

		/* If usage model is split task */
		if(ACCUsageModel==SPLIT_TASK)
		{
			/* Create FIRA Tasks */
			eFirResult = adi_fir_CreateTask(hFir[0], /* Device Handle*/
					FirTaskChannels,              /* Pointer to Channel List */
					(NCH_ACC)/2,					  	  /* Number of Channels  */
					&FirTaskMemory[0],               /* Pointer to Memory */
					FIR_MEM_SIZE((NCH_ACC)/2),           	/*Memory Size */
					&hFirTask[0]);               /* Address to store the handle */
			CHECK_FIR_RESULT(eFirResult)

			eFirResult = adi_fir_CreateTask(hFir[1], /* Device Handle*/
					&FirTaskChannels[NCH_ACC/2],      /* Pointer to Channel List */
					NCH_ACC/2,					  	  /* Number of Channels  */
					&FirTaskMemory[1],               /* Pointer to Memory */
					FIR_MEM_SIZE(NCH_ACC/2),       	/*Memory Size */
					&hFirTask[1]);               	/* Address to store the handle */
		}
		else
		{

			/* Create FIRA Tasks */
			eFirResult = adi_fir_CreateTask(hFir[0], /* Device Handle*/
					FirTaskChannels,              /* Pointer to Channel List */
					NCH/2,					  	  /* Number of Channels  */
					&FirTaskMemory[0],               /* Pointer to Memory */
					FIR_MEM_SIZE(NCH/2),           	/*Memory Size */
					&hFirTask[0]);               /* Address to store the handle */
			CHECK_FIR_RESULT(eFirResult)

			eFirResult = adi_fir_CreateTask(hFir[1], /* Device Handle*/
					&FirTaskChannels[NCH/2],       /* Pointer to Channel List */
					NCH/2,					  	  /* Number of Channels  */
					&FirTaskMemory[1],               /* Pointer to Memory */
					FIR_MEM_SIZE(NCH/2),           	/*Memory Size */
					&hFirTask[1]);               /* Address to store the handle */
			CHECK_FIR_RESULT(eFirResult)
		}

		/* Create IIR Accelerator Tasks */
		for(i=0;i<NCH;i++)
		{
			if(i<4)
			{
				eIirResult = adi_iir_CreateTask(hIir[i], 			  /* Device Handle */
										IirTaskChannels[i],           /* Pointer to Channel List */
										5, 							  /* Number of Channels */
										&IirTaskMemory[i],            /* Pointer to Task Memory */
										IIR_MEM_SIZE(IIR_BANDS),      /* Memory Size */
										&hIirTasks[i]);               /* Address to store handle */
				CHECK_IIR_RESULT(eIirResult);
			}
			if(i>=4&&i<8)
			{
				eIirResult = adi_iir_CreateTask(hIir[i-4], 				/* Device Handle */
										IirTaskChannels[i],           /* Pointer to Channel List */
										5, 							  /* Number of Channels */
										&IirTaskMemory[i],            /* Pointer to Task Memory */
										IIR_MEM_SIZE(IIR_BANDS),      /* Memory Size */
										&hIirTasks[i]);               /* Address to store handle */
				CHECK_IIR_RESULT(eIirResult);
			}
			if(i>=8&&i<12)
			{
				eIirResult = adi_iir_CreateTask(hIir[i-8], 				/* Device Handle */
										IirTaskChannels[i],           /* Pointer to Channel List */
										5, 							  /* Number of Channels */
										&IirTaskMemory[i],            /* Pointer to Task Memory */
										IIR_MEM_SIZE(IIR_BANDS),      /* Memory Size */
										&hIirTasks[i]);               /* Address to store handle */
				CHECK_IIR_RESULT(eIirResult);
			}
		}
	}

	/* Reinitialize SPORTs */
	if(init_flag!=0)
	{
		//Sport_Init();
	}
	else
	{
		init_flag = 1;
	}
}

/* FIR/IIR processing */
void Process_FIR_IIR(ACC_USAGE_MODEL ACCUsageModel)
{
																	START_CYCLE_COUNT
	int i, j; float gains[5];

	/* FIR result */
	ADI_FIR_RESULT eFirResult = ADI_FIR_RESULT_SUCCESS;

	/* IIR result */
	ADI_IIR_RESULT eIirResult = ADI_IIR_RESULT_SUCCESS;

	/* Don't do any processing if frame work needs to be reinitialized */
	if(FWReInit==1)
	{
		return;
	}

	/* Reset the task done counters */
	FIRTaskDone = 0;
	IIRTaskDoneCount=0;

	/* Core Only Usage model */
	if(ACCUsageModel==CORE_ONLY)
	{

		adi_osal_EnterCriticalRegion();

		/*FIR processing*/
		for(i=0;i<NCH;i++)
		{

			//fir((float*)&Input[i][0], (float*)&Output[i], (pm float *)(&FIRCoeffs), (float*)&fir_state[i], BLOCK_SIZE, FIR_TAPS);
			 firf((float*)&Input[i][0], (float*)&Output[i], BLOCK_SIZE, &firobj[i]);
		}

		/* IIR processing */
		for(i=0;i<NCH;i++)
		{
#if 0
			/* IIR processing */
			biquad((float*)&Output[i], &IIROutput[i][0], &IIRCoeffs[i-NCH_ACC][0], &iir_state[i-NCH_ACC][0], BLOCK_SIZE, NSECTIONS);
			biquad((float*)&Output[i], &IIROutput[i][BLOCK_SIZE], &IIRCoeffs[i-NCH_ACC][NSECTIONS*5], &iir_state[i-NCH_ACC][NSECTIONS*2], BLOCK_SIZE, NSECTIONS);
			biquad((float*)&Output[i], &IIROutput[i][2*BLOCK_SIZE], &IIRCoeffs[i-NCH_ACC][2*NSECTIONS*5], &iir_state[i-NCH_ACC][2*NSECTIONS*2], BLOCK_SIZE, NSECTIONS);
			biquad((float*)&Output[i], &IIROutput[i][3*BLOCK_SIZE], &IIRCoeffs[i-NCH_ACC][3*NSECTIONS*5], &iir_state[i-NCH_ACC][3*NSECTIONS*2], BLOCK_SIZE, NSECTIONS);
			biquad((float*)&Output[i], &IIROutput[i][4*BLOCK_SIZE], &IIRCoeffs[i-NCH_ACC][4*NSECTIONS*5], &iir_state[i-NCH_ACC][4*NSECTIONS*2], BLOCK_SIZE, NSECTIONS);
#else
				s_iirf_df2 ((s_iirf_df2_t *)&iirobj[i][0], &IIROutput[i][0], &Output[i], BLOCK_SIZE);
				s_iirf_df2 ((s_iirf_df2_t *)&iirobj[i][1], &IIROutput[i][BLOCK_SIZE], &Output[i], BLOCK_SIZE);
				s_iirf_df2 ((s_iirf_df2_t *)&iirobj[i][2], &IIROutput[i][2*BLOCK_SIZE], &Output[i], BLOCK_SIZE);
				s_iirf_df2 ((s_iirf_df2_t *)&iirobj[i][3], &IIROutput[i][3*BLOCK_SIZE], &Output[i], BLOCK_SIZE);
				s_iirf_df2 ((s_iirf_df2_t *)&iirobj[i][4], &IIROutput[i][4*BLOCK_SIZE], &Output[i], BLOCK_SIZE);
		}
	    adi_osal_ExitCriticalRegion();

#endif
	}

	/* Direct Replacement usage model */
	else if(ACCUsageModel==DIRECT_REPLACEMENT)
	{

		cc = clock();

		flush_data_buffer(&Input[0][0],&Input[NCH-1][CIRCULAR_BUFFER_LENGTH-1],0);

		flush_data_buffer(&IIROutput[0][0],&IIROutput[NCH-1][BLOCK_SIZE*IIR_BANDS-1],1);

		flush_data_buffer(&FIRCoeffs[0],&FIRCoeffs[FIR_TAPS-1],1);

		cache_cycles = clock() - cc;

		/* Queue FIR tasks */
		eFirResult = adi_fir_QueueTask(hFirTask[0]);
		CHECK_FIR_RESULT(eFirResult);

		eFirResult = adi_fir_QueueTask(hFirTask[1]);
		CHECK_FIR_RESULT(eFirResult);
																	TAKE_CYCLE_SNAPSHOT
																	ADD_TO_CORE_CYCLES

		/* Wait till FIR tasks are done */
		while(FIRTaskDone<FIRA_INSTANCES);
	    FIRTaskDone = 0;
																	TAKE_CYCLE_SNAPSHOT
																	ADD_TO_CORE_CYCLES
																	ADD_TO_FIRA_CYCLES

		/* Queue IIR tasks */
		for(i=0;i<NCH;i++)
		{
			eIirResult = adi_iir_QueueTask(hIirTasks[i]);
			CHECK_IIR_RESULT(eIirResult);
		}

																	TAKE_CYCLE_SNAPSHOT
																	ADD_TO_CORE_CYCLES

		/* Wait till IIR tasks are done */
		while(IIRTaskDoneCount<NCH);
		IIRTaskDoneCount=0;
																	TAKE_CYCLE_SNAPSHOT
																	ADD_TO_CORE_CYCLES
																	ADD_TO_IIRA_CYCLES

	}

	/* Split Task usage model */
	else if(ACCUsageModel==SPLIT_TASK)
	{
		cc = clock();

		flush_data_buffer(&Input[0][0],&Input[NCH_ACC-1][CIRCULAR_BUFFER_LENGTH-1],0);

		flush_data_buffer(&IIROutput[0][0],&IIROutput[NCH_ACC-1][BLOCK_SIZE*IIR_BANDS-1],1);

		flush_data_buffer(&FIRCoeffs[0],&FIRCoeffs[FIR_TAPS-1],1);

		cache_cycles = cc - clock();

		TAKE_CYCLE_SNAPSHOT
		/* Queue FIR task */												ADD_TO_CORE_CYCLES
		eFirResult = adi_fir_QueueTask(hFirTask[0]);
		eFirResult = adi_fir_QueueTask(hFirTask[1]);
																			TAKE_CYCLE_SNAPSHOT
																			ADD_TO_CORE_CYCLES

adi_osal_EnterCriticalRegion();

		/* Start core processing */
		for(i=NCH_ACC;i<NCH;i++)
		{
			/*FIR processing*/
			//fir((float*)&Input[i][0], (float*)&Output[i], (float *)(&FIRCoeffs), (float*)&Input[i][BLOCK_SIZE], BLOCK_SIZE, FIR_TAPS);
			firf((float*)&Input[i][0], (float*)&Output[i], BLOCK_SIZE, &firobj[i]);
		}

		for(i=NCH_ACC;i<NCH;i++)
		{
#if 0
			/* IIR processing */
			biquad((float*)&Output[i], &IIROutput[i][0], &IIRCoeffs[i-NCH_ACC][0], &iir_state[i-NCH_ACC][0], BLOCK_SIZE, NSECTIONS);
			biquad((float*)&Output[i], &IIROutput[i][BLOCK_SIZE], &IIRCoeffs[i-NCH_ACC][NSECTIONS*5], &iir_state[i-NCH_ACC][NSECTIONS*2], BLOCK_SIZE, NSECTIONS);
			biquad((float*)&Output[i], &IIROutput[i][2*BLOCK_SIZE], &IIRCoeffs[i-NCH_ACC][2*NSECTIONS*5], &iir_state[i-NCH_ACC][2*NSECTIONS*2], BLOCK_SIZE, NSECTIONS);
			biquad((float*)&Output[i], &IIROutput[i][3*BLOCK_SIZE], &IIRCoeffs[i-NCH_ACC][3*NSECTIONS*5], &iir_state[i-NCH_ACC][3*NSECTIONS*2], BLOCK_SIZE, NSECTIONS);
			biquad((float*)&Output[i], &IIROutput[i][4*BLOCK_SIZE], &IIRCoeffs[i-NCH_ACC][4*NSECTIONS*5], &iir_state[i-NCH_ACC][4*NSECTIONS*2], BLOCK_SIZE, NSECTIONS);
#else
			s_iirf_df2 ((s_iirf_df2_t *)&iirobj[i][0], &IIROutput[i][0], &Output[i], BLOCK_SIZE);
			s_iirf_df2 ((s_iirf_df2_t *)&iirobj[i][1], &IIROutput[i][BLOCK_SIZE], &Output[i], BLOCK_SIZE);
			s_iirf_df2 ((s_iirf_df2_t *)&iirobj[i][2], &IIROutput[i][2*BLOCK_SIZE], &Output[i], BLOCK_SIZE);
			s_iirf_df2 ((s_iirf_df2_t *)&iirobj[i][3], &IIROutput[i][3*BLOCK_SIZE], &Output[i], BLOCK_SIZE);
			s_iirf_df2 ((s_iirf_df2_t *)&iirobj[i][4], &IIROutput[i][4*BLOCK_SIZE], &Output[i], BLOCK_SIZE);

#endif
		}
adi_osal_ExitCriticalRegion();

		/* Wait for the accelerator processing to be over */
		while(IIRTaskDoneCount<NCH_ACC);
		IIRTaskDoneCount=0;
	}

	float Scale = 1.0;

	/* Multiply IIR output for different bands with their respective gains and add to get the final output */
	for(i=0;i<NCH;i++)
	{
		for(j=0;j<BLOCK_SIZE;j++)
		{
			Output[i][j] = IIROutput[i][j]*IIRGainsFinal[i][0]+IIROutput[i][BLOCK_SIZE+j]*IIRGainsFinal[i][1]+IIROutput[i][2*BLOCK_SIZE+j]*IIRGainsFinal[i][2]+IIROutput[i][3*BLOCK_SIZE+j]*IIRGainsFinal[i][3]+IIROutput[i][4*BLOCK_SIZE+j]*IIRGainsFinal[i][4];
		}
	}

																		TAKE_CYCLE_SNAPSHOT
																		ADD_TO_CORE_CYCLES
    /* Data Pipelining usage model */
    if(ACCUsageModel==DATA_PIPELINING)
	{
		cc = clock();

		flush_data_buffer(&Input[0][0],&Input[NCH-1][CIRCULAR_BUFFER_LENGTH-1],0);

		flush_data_buffer(&IIROutput[0][0],&IIROutput[NCH-1][BLOCK_SIZE*IIR_BANDS-1],1);

		flush_data_buffer(&FIRCoeffs[0],&FIRCoeffs[FIR_TAPS-1],1);

		cache_cycles = cc - clock();

		/* Queue FIR task and come out, no need to wait for the accelerator processing done, IIR tasks will be queued inside FIR */
		eFirResult = adi_fir_QueueTask(hFirTask[0]);
		eFirResult = adi_fir_QueueTask(hFirTask[1]);
																				TAKE_CYCLE_SNAPSHOT
																				ADD_TO_CORE_CYCLES
	}
	else
	{
																				CALCULATE_MIPS
	}

}

/* Initialize FIR task channels */
void InitializeFIRTaskChannels(int nch)
{
	int i;

	for(i=0; i<nch; i++)
	{
		FirTaskChannels[i].nTapLength 			= 	FIR_TAPS;
		FirTaskChannels[i].nWindowSize 			= 	BLOCK_SIZE;
		FirTaskChannels[i].eSampling 			= 	ADI_FIR_SAMPLING_SINGLE_RATE;
		FirTaskChannels[i].nSamplingRatio 		= 	1;
		FirTaskChannels[i].nCoefficientCount 	= 	FIR_TAPS;
		FirTaskChannels[i].nCoefficientModify 	= 	1;
		FirTaskChannels[i].pCoefficientIndex 	= 	(void*)&FIRCoeffs;
		FirTaskChannels[i].pOutputBuffBase 		= 	(void*)&FIROutput[i];
		FirTaskChannels[i].nOutputBuffCount 	= 	BLOCK_SIZE;
		FirTaskChannels[i].nOutputBuffModify 	= 	1;
		FirTaskChannels[i].pOutputBuffIndex 	= 	(void*)&FIROutput[i];
		FirTaskChannels[i].pInputBuffBase 		= 	(void*)&Input[i];
		FirTaskChannels[i].nInputBuffModify		= 	1;
		FirTaskChannels[i].nInputBuffCount 		= 	CIRCULAR_BUFFER_LENGTH;
		FirTaskChannels[i].pInputBuffIndex 		= 	(void*)&Input[i][FIRA_INDEX_START];
	}
}

/* Initialize IIR task channels */
void InitializeIIRTaskChannels(int nch)
{
	int i, j;

	for(i=0; i<nch; i++)
	{
		for(j=0; j<IIR_BANDS; j++)
		{
			IirTaskChannels[i][j].nBiquads 				= 	NSECTIONS;
			IirTaskChannels[i][j].nWindowSize 			= 	BLOCK_SIZE;
			IirTaskChannels[i][j].nCoefficientLength 	= 	7*NSECTIONS;
			IirTaskChannels[i][j].nCoefficientModify 	= 	1;
			IirTaskChannels[i][j].pCoefficientIndex 	= 	(void*)&IIRCoeffsACC[i][j*NSECTIONS*7];
			IirTaskChannels[i][j].pOutputBuffBase 		= 	(void*)&IIROutput[i][j*BLOCK_SIZE];
			IirTaskChannels[i][j].nOutputBuffLength 	= 	BLOCK_SIZE;
			IirTaskChannels[i][j].nOutputBuffModify 	= 	1;
			IirTaskChannels[i][j].pOutputBuffIndex 		= 	(void*)&IIROutput[i][j*BLOCK_SIZE];
			IirTaskChannels[i][j].pInputBuffBase 		= 	(void*)&FIROutput[i];
			IirTaskChannels[i][j].nInputBuffModify 		= 	1;
			IirTaskChannels[i][j].nInputBuffLength 		= 	BLOCK_SIZE;
			IirTaskChannels[i][j].pInputBuffIndex 		= 	(void*)&FIROutput[i];
		}
	}
}


/* FIR task done callback */
void FIRTaskDoneCallback( void * pCBParam, ADI_FIR_EVENT Event, void * pArg)
{
	switch(Event)
	{
	    case ADI_FIR_EVENT_ALL_CHANNEL_DONE:
	    	FIRTaskDone++;

	    	if(FIRTaskDone==FIRA_INSTANCES)
	    	{
				/* Queue IIR tasks for the split task model */
				if(ACCUsageModel==SPLIT_TASK)
				{
																			TAKE_CYCLE_SNAPSHOT
																			ADD_TO_CORE_CYCLES
																			ADD_TO_FIRA_CYCLES
						for(int i=0;i<NCH_ACC;i++)
						{
							adi_iir_QueueTask(hIirTasks[i]);
						}
																			TAKE_CYCLE_SNAPSHOT
																			ADD_TO_CORE_CYCLES
				}
				/* Queue IIR tasks for the data pipelining model */
				else if(ACCUsageModel==DATA_PIPELINING)
				{															TAKE_CYCLE_SNAPSHOT
																			ADD_TO_FIRA_CYCLES

						for(int i=0;i<NCH;i++)
						{
							adi_iir_QueueTask(hIirTasks[i]);
						}														TAKE_CYCLE_SNAPSHOT
				}
	    	}
	}
}

/* IIR task done callback */
void IIRTaskDoneCallback( void * pCBParam, ADI_IIR_EVENT Event, void * pArg)
{
																		TAKE_CYCLE_SNAPSHOT
																		if(ACCUsageModel!=DATA_PIPELINING)
																		{
																			ADD_TO_CORE_CYCLES
																		}
																		ADD_TO_IIRA_CYCLES

																		if(ACCUsageModel==DATA_PIPELINING)
																		{
																			CALCULATE_MIPS
																		}
	switch(Event)
	{
	    case ADI_IIR_EVENT_ALL_CHANNEL_DONE:
	    	IIRTaskDoneCount++;
	}
}


