#ifndef __PROCESSING_H__
#define __PROCESSING_H__

#include <sys/platform.h>
#include "adi_initialize.h"
#include <drivers/fir/adi_fir.h>
#include <drivers/iir/adi_iir.h>
#include "stdio.h"
#include "stdlib.h"
#include "time.h"
#include "filter.h"
#include "uart.h"
#include "string.h"
#include <filter.h>
#include <adi_filter.h>

#define BLOCK_SIZE 256									/* Block Size */
#define CIRCULAR_BUFFER_LENGTH 1536						/* Circular buffer length for FIR - should be greater than of equal to  BLOCK_SIZE+FIR_TAPS+1  and multiple of BLOCK_SIZE*/
#define WP_START   CIRCULAR_BUFFER_LENGTH-BLOCK_SIZE	/* Write pointer start, start filling the delay line buffer from this pointer onwards and increment the write pointer by BLOCK_SIZE for each new block */
#define FIRA_INDEX_START (WP_START-FIR_TAPS+1)			/* FIRA start address should be initialized with this value */
#define NCH		   12									/* Total number of channels */
#define FIR_TAPS   1024									/* FIR TAPS */
#define IIR_BANDS  5									/* IIR bands */
#define NSECTIONS  4									/* Number of biquad stages per IIR band */
#define NCH_CORE   6									/* Number of channels to be processed by core */
#define NCH_ACC	   6									/* Number of channles to be processed by accelerator */


#define FIRA_INSTANCES	2
#define IIRA_INSTANCES	4

/* Macro for checking Result returned by the driver after API call */
#define CHECK_FIR_RESULT(eResult) \
        if(eResult != ADI_FIR_RESULT_SUCCESS)\
		{\
            printf("CHECK_RESULT failed at line %d of file %s \n",__LINE__,__FILE__);\
            exit(-2);\
       }
/* Macro for checking Result returned by the driver after API call */
#define CHECK_IIR_RESULT(eResult) \
        if(eResult != ADI_IIR_RESULT_SUCCESS)\
		{\
            printf("CHECK_RESULT failed at line %d of file %s \n",__LINE__,__FILE__);\
            exit(-2);\
        }

extern float Input[NCH][CIRCULAR_BUFFER_LENGTH];
extern float Output[NCH][BLOCK_SIZE];
extern float iir_state[NCH][2*NSECTIONS*IIR_BANDS];

extern volatile unsigned int wp;

/* Function for one time initialization required for FIR/IIR processing */
void Init_FIR_IIR(ACC_USAGE_MODEL ACCUsageModel);

/* Function for FIR/IIR processing */
void Process_FIR_IIR(ACC_USAGE_MODEL ACCUsageModel);
#endif
