/********************************************************************
 *                                                                  *
 * THIS FILE IS PART OF THE OggVorbis SOFTWARE CODEC SOURCE CODE.   *
 * USE, DISTRIBUTION AND REPRODUCTION OF THIS LIBRARY SOURCE IS     *
 * GOVERNED BY A BSD-STYLE SOURCE LICENSE INCLUDED WITH THIS SOURCE *
 * IN 'COPYING'. PLEASE READ THESE TERMS BEFORE DISTRIBUTING.       *
 *                                                                  *
 * THE OggVorbis SOURCE CODE IS (C) COPYRIGHT 1994-2009             *
 * by the Xiph.Org Foundation https://xiph.org/                     *
 *                                                                  *
 ********************************************************************

 function: simple example decoder

 ********************************************************************/

/* Takes a vorbis bitstream from stdin1 and writes raw stereo PCM to
   stdout1. Decodes simple and chained OggVorbis files from beginning
   to end. Vorbisfile.a is somewhat more complex than the code below.  */

/* Note that this is POSIX, not ANSI code */

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <Vorbis/codec.h>
#include "Audio_Talkthrough_I2S.h"
#include "Processing.h"
#include "uart.h"

#ifdef _WIN32 /* We need the following two to set stdin1/stdout1 to binary */
#include <io.h>
#include <fcntl.h>
#endif

#if defined(__MACOS__) && defined(__MWERKS__)
#include <console.h>      /* CodeWarrior's Mac "command-line" support */
#endif

ogg_int16_t convbuffer[4096]; /* take 8k out of the data segment, not the stack */
int convsize=4096;

extern void _VDBG_dump(void);

FILE *stdin1;
FILE *stdout1;

#ifdef ADI_DEBUG

#include "time.h"

unsigned int cycles1;
unsigned int cycles1_per_frame=0;
float MIPS;
float Peak_MIPS;
float Avg_MIPS=0;
float sample_rate;
int frame_cnt=0;

extern volatile int DecRCnt;
extern volatile int DecWCnt;

#define BOUT_BUFFSIZE 100
volatile int BoutBuff[BOUT_BUFFSIZE];
volatile int BoutBuffIndex=0;

//#define PRINT_cycles1

#ifdef PRINT_cycles1
#define PRINTC printf
#else
#define PRINTC
#endif

#endif

#include "string.h"

//#define FILE_SIZE 4458692//265589
//#define FILE_SIZE 265589
#define FILE_SIZE 2176447//2910793 /* New day 1*/
//#define FILE_SIZE 178958//2910793 /* Scamm */
//#define FILE_SIZE 2594914//178958//2910793 /* Dreams */
//#define FILE_SIZE 3165309//178958//2910793 /* Aashaayen */
//#define FILE_SIZE 4616600//178958//2910793 /* Ek_Pal_Ka_Jeena */
//#define FILE_SIZE 2742086//2910793 /* Lose_Control*/


//#pragma section("seg_sdram")
unsigned char __attribute__ ((section (".L3.data"))) OggFileBuff[FILE_SIZE]=
{
	//#include "../IO_Files/sample9in.dat"
	//#include "../IO_Files/music1in.dat"
	//#include "../IO_Files/music2in.dat"
	#include "../IO_Files/newday1in.dat"
	//#include "../IO_Files/Scaamin.dat"
	//#include "../IO_Files/Dreamsin.dat"
	//#include "../IO_Files/Aashaayenin.dat"
	//#include "../IO_Files/Ek_Pal_Ka_Jeenain.dat"
	//#include "../IO_Files/Lose_Controlin.dat"
};
volatile unsigned char *pReadPointer=0;
volatile int ReadCnt=0;

#ifdef ADI_DEBUG
 
//#define SAMPLE1
//#define SAMPLE2
//#define SAMPLE3
//#define SAMPLE4
//#define SAMPLE5
//#define SAMPLE6
//#define SAMPLE7
//#define SAMPLE8
#define SAMPLE9

#ifdef SAMPLE1
	#define INPUT_FILE "../IO_Files/sample1.ogg"
	#define OUTPUT_FILE "../IO_Files/sample1.dat"
#endif

#ifdef SAMPLE2
	#define INPUT_FILE "../IO_Files/sample2.ogg"
	#define OUTPUT_FILE "../IO_Files/sample2.dat"
#endif

#ifdef SAMPLE3
	#define INPUT_FILE "../IO_Files/sample3.ogg"
	#define OUTPUT_FILE "../IO_Files/sample3.dat"
#endif

  #ifdef SAMPLE4
	#define INPUT_FILE "../IO_Files/sample4.ogg"
	#define OUTPUT_FILE "../IO_Files/sample4.dat"
#endif

#ifdef SAMPLE5
	#define INPUT_FILE "../IO_Files/sample5.ogg"
	#define OUTPUT_FILE "../IO_Files/sample5.dat"
#endif

#ifdef SAMPLE6
	#define INPUT_FILE "../IO_Files/sample6.ogg"
	#define OUTPUT_FILE "../IO_Files/sample6.dat"
#endif

#ifdef SAMPLE7
	#define INPUT_FILE "../IO_Files/sample7.ogg"
	#define OUTPUT_FILE "../IO_Files/sample7.dat"
#endif

#ifdef SAMPLE8
	#define INPUT_FILE "../IO_Files/sample8.ogg"
	#define OUTPUT_FILE "../IO_Files/sample8.dat"
#endif

#ifdef SAMPLE9
	#define INPUT_FILE "../IO_Files/sample9.ogg"
	#define OUTPUT_FILE "../IO_Files/sample9.dat"
#endif

#endif


extern int   DACBuffer[];
extern volatile int DecRIndex;
extern volatile int DecWIndex;

/* Flag for synchronization between core and SPORT DMA */
extern volatile bool  BlockReady;

/* SPU handle */
static ADI_SPU_HANDLE      ghSpu;


extern void Process_Block(void);

void OggVorbisDecode(void);

#define OPF_CNT_SIZE 100
volatile unsigned int OPFCycles=0;
volatile unsigned int DecodeCnt=0;
volatile unsigned int DecodeCycles=0;
volatile unsigned int OPFCnt=0;
volatile unsigned int OPFTotalCycles[OPF_CNT_SIZE];
volatile unsigned long long OPFDecoderCycles[OPF_CNT_SIZE];
volatile unsigned int OPFPPCycles[OPF_CNT_SIZE];
float DecoderMIPS=0;

void OggVorbisDecode(void)
{

	ogg_sync_state   oy; /* sync and verify incoming physical bitstream */
	ogg_stream_state os; /* take physical pages, weld into a logical
	                        stream of packets */
	ogg_page         og; /* one Ogg bitstream page. Vorbis packets are inside */
	ogg_packet       op; /* one raw packet of data for decode */

	vorbis_info      vi; /* struct that stores all the static vorbis bitstream
	                        settings */
	vorbis_comment   vc; /* struct that stores all the bitstream user comments */
	vorbis_dsp_state vd; /* central working state for the packet->PCM decoder */
	vorbis_block     vb; /* local working space for packet->PCM decode */

	char *buffer;
	int  bytes;

 // stdin1 = fopen(INPUT_FILE, "rb");
  //stdout1 = fopen(OUTPUT_FILE, "wb");

  //_setmode( _fileno( stdin1 ), _O_BINARY );
  /********** Decode setup ************/

  ogg_sync_init(&oy); /* Now we can read pages */

  pReadPointer = OggFileBuff;

  while(1)
  { /* we repeat if the bitstream is chained */
    int eos=0;
    int i;

    /* grab some data at the head of the stream. We want the first page
       (which is guaranteed to be small and only contain the Vorbis++
       stream initial header) We need the first page to get the stream
       serialno. */

    /* submit a 4k block to libvorbis' Ogg layer */
    buffer=ogg_sync_buffer(&oy,4096);
#if 0
    bytes=fread(buffer,1,4096,stdin1);

#else
    if(FILE_SIZE-ReadCnt>4096)
    {
    	bytes = 4096;
    	memcpy((void *)buffer, (const void *)pReadPointer, bytes);
    	pReadPointer+=4096;
    	ReadCnt+=4096;
    }
    else
    {
    	bytes=0;
    }
#endif

    ogg_sync_wrote(&oy,bytes);

    /* Get the first page. */
    if(ogg_sync_pageout(&oy,&og)!=1){
      /* have we simply run out of data?  If so, we're done. */
      if(bytes<4096)break;

      /* error case.  Must not be Vorbis data */
      fprintf(stderr,"Input does not appear to be an Ogg bitstream.\n");
      exit(1);
    }

    /* Get the serial number and set up the rest of decode. */
    /* serialno first; use it to set up a logical stream */
    ogg_stream_init(&os,ogg_page_serialno(&og));

    /* extract the initial header from the first page and verify that the
       Ogg bitstream is in fact Vorbis data */

    /* I handle the initial header first instead of just having the code
       read all three Vorbis headers at once because reading the initial
       header is an easy way to identify a Vorbis bitstream and it's
       useful to see that functionality seperated out. */

    vorbis_info_init(&vi);
    vorbis_comment_init(&vc);
    if(ogg_stream_pagein(&os,&og)<0){
      /* error; stream version mismatch perhaps */
      fprintf(stderr,"Error reading first page of Ogg bitstream data.\n");
      exit(1);
    }

    if(ogg_stream_packetout(&os,&op)!=1){
      /* no page? must not be vorbis */
      fprintf(stderr,"Error reading initial header packet.\n");
      exit(1);
    }

    if(vorbis_synthesis_headerin(&vi,&vc,&op)<0){
      /* error case; not a vorbis header */
      fprintf(stderr,"This Ogg bitstream does not contain Vorbis "
              "audio data.\n");
      exit(1);
    }

    /* At this point, we're sure we're Vorbis. We've set up the logical
       (Ogg) bitstream decoder. Get the comment and codebook headers and
       set up the Vorbis decoder */

    /* The next two packets in order are the comment and codebook headers.
       They're likely large and may span multiple pages. Thus we read
       and submit data until we get our two packets, watching that no
       pages are missing. If a page is missing, error out; losing a
       header page is the only place where missing data is fatal. */

    i=0;
    while(i<2){ //1
      while(i<2){ //2
        int result=ogg_sync_pageout(&oy,&og);
        if(result==0)break; /* Need more data */
        /* Don't complain about missing or corrupt data yet. We'll
           catch it at the packet output phase */
        if(result==1){ //3
          ogg_stream_pagein(&os,&og); /* we can ignore any errors here
                                         as they'll also become apparent
                                         at packetout */
          while(i<2){//4
            result=ogg_stream_packetout(&os,&op);
            if(result==0)break;
            if(result<0){//5
              /* Uh oh; data at some point was corrupted or missing!
                 We can't tolerate that in a header.  Die. */
              fprintf(stderr,"Corrupt secondary header.  Exiting.\n");
              exit(1);
            }//5
            result=vorbis_synthesis_headerin(&vi,&vc,&op);
            if(result<0){
              fprintf(stderr,"Corrupt secondary header.  Exiting.\n");
              exit(1);
            }//4
            i++;
          }//3
        }//2
      }//1
      /* no harm in not checking before adding more */
      buffer=ogg_sync_buffer(&oy,4096);
#if 0
    bytes=fread(buffer,1,4096,stdin1);

#else
    if(FILE_SIZE-ReadCnt>4096)
    {
    	bytes = 4096;
    	memcpy((void *)buffer, (const void *)pReadPointer, bytes);
    	pReadPointer+=4096;
    	ReadCnt+=4096;
    }
    else
    {
    	bytes = FILE_SIZE-ReadCnt;
    	memcpy((void *)buffer, (const void *)pReadPointer, bytes);
    }
#endif
      if(bytes==0 && i<2){
        fprintf(stderr,"End of file before finding all Vorbis headers!\n");
        exit(1);
      }
      ogg_sync_wrote(&oy,bytes);
    }

    /* Throw the comments plus a few lines about the bitstream we're
       decoding */
    {
      char **ptr=vc.user_comments;
      while(*ptr){
        //fprintf(stderr,"%s\n",*ptr);
        ++ptr;
      }
     // fprintf(stderr,"\nBitstream is %d channel, %ldHz\n",vi.channels,vi.rate);
    //  fprintf(stderr,"Encoded by: %s\n\n",vc.vendor);
    }

    convsize=4096/vi.channels;

    /* OK, got and parsed all three headers. Initialize the Vorbis
       packet->PCM decoder. */
    if(vorbis_synthesis_init(&vd,&vi)==0){ /* central decode state */
      vorbis_block_init(&vd,&vb);          /* local state for most of the decode
                                              so multiple block decodes can
                                              proceed in parallel. We could init
                                              multiple vorbis_block structures
                                              for vd here */
      /* The rest is just a straight decode loop until end of stream */
      while(!eos){
        while(!eos){
        	cycles1 = clock();
          int result=ogg_sync_pageout(&oy,&og);
          cycles1 = clock()-cycles1;
          PRINTC("\nogg_sync_pageout cycles1=%d",cycles1);
          cycles1_per_frame+=cycles1;

          if(result==0)break; /* need more data */
          if(result<0){ /* missing or corrupt data at this page position */
           // fprintf(stderr,"Corrupt or missing data in bitstream; "
                    //"continuing...\n");
          }else{
        	  cycles1 = clock();
            ogg_stream_pagein(&os,&og); /* can safely ignore errors at
                                           this point */
            cycles1 = clock()-cycles1;
                    PRINTC("\n\nogg_stream_pagein cycles1=%d",cycles1);
            cycles1_per_frame+=cycles1;

            while(1){
            	 cycles1 = clock();
              result=ogg_stream_packetout(&os,&op);
              cycles1 = clock()-cycles1;
                  PRINTC("\nogg_stream_packetout cycles1=%d",cycles1);
                  cycles1_per_frame+=cycles1;


              if(result==0)break; /* need more data */
              if(result<0){ /* missing or corrupt data at this page position */
                /* no reason to complain; already complained above */
              }else{
                /* we have a packet.  Decode it */
                float **pcm;
                int samples;

                cycles1 = clock();
                if(vorbis_synthesis(&vb,&op)==0) /* test for success! */
                {
                  vorbis_synthesis_blockin(&vd,&vb);
                  cycles1 = clock()-cycles1;

                }
                cycles1_per_frame+=cycles1;
                PRINTC("\nvorbis_synthesis and vorbis_synthesis_blockin cycles1=%d",cycles1);

                /*

                **pcm is a multichannel float vector.  In stereo, for
                example, pcm[0] is left, and pcm[1] is right.  samples is
                the size of each channel.  Convert the float values
                (-1.<=range<=1.) to whatever PCM format and write it out */

                while(1)
                {

                  cycles1=clock();
                  (samples=vorbis_synthesis_pcmout(&vd,&pcm));
                  cycles1=clock()-cycles1;
                  cycles1_per_frame+=cycles1;
                  PRINTC("\nvorbis_synthesis_pcmout cycles1=%d",cycles1);


                  if(samples<=0)
                	  break;

                  MIPS=(float)cycles1_per_frame*(float)vi.rate;

                  MIPS=(MIPS*0.000001)/(float)samples;


                  if(frame_cnt!=0)
                  {
                	  if(MIPS>Peak_MIPS)
                		  	 Peak_MIPS=MIPS;

                	  Avg_MIPS=(Avg_MIPS*(frame_cnt-1)+MIPS)/frame_cnt;
                  }

                  //MIPS=((float)(cycles1_per_frame*vi.rate)/(float)samples)*0.000001f;
                  PRINTC("\nframe_no=%d, cycles1_per_frame=%d, samples=%d, MIPS=%f, Peak MIPS=%0.2f, Average MIPS=%0.2f",frame_cnt, cycles1_per_frame, samples, MIPS, Peak_MIPS, Avg_MIPS);
                  cycles1_per_frame=0;
                  frame_cnt++;
#ifdef PRINT_cycles1
                  if(frame_cnt==51)
                	    exit(1);
#endif
                  cycles1=clock();
                  int j;
                  int clipflag=0;
                  int bout=(samples<convsize?samples:convsize);

                  /* convert floats to 16 bit signed ints (host order) and
                     interleave */
                  for(i=0;i<vi.channels;i++){
                    ogg_int16_t *ptr=convbuffer+i;
                    float  *mono=pcm[i];
                    for(j=0;j<bout;j++){
#if 1
                      int val=floor(mono[j]*32767.f+.5f);
#else /* optional dither */
                      int val=mono[j]*32767.f+drand48()-0.5f;
#endif
                      /* might as well guard against clipping */
                      if(val>32767){
                        val=32767;
                        clipflag=1;
                      }
                      if(val<-32768){
                        val=-32768;
                        clipflag=1;
                      }
                      *ptr=val;
                      ptr+=vi.channels;
                    }
                 }
                  cycles1=clock()-cycles1;
                  PRINTC("\nfloat to fix cycles1=%d",cycles1);


                  //if(clipflag)
                   // fprintf(stderr,"Clipping in frame %ld\n",(long)(vd.sequence));


#if 0
                  fwrite(convbuffer,2*vi.channels,bout,stdout1);
#else
                  //printf("\n\nbout = %d", bout);
                 // printf("\n\nbout = %d", bout);

                  for(i=0; i<2*bout; i++)
                  {
                	  DACBuffer[DecWIndex++]=((int)convbuffer[i]<<4);
                	  DecWIndex=DecWIndex%(2*DECODE_SIZE);
                  }

                  BoutBuff[BoutBuffIndex++]=bout;
                  BoutBuffIndex=BoutBuffIndex%BOUT_BUFFSIZE;

                  while(1)
                  {
                	  if(DecodeCycles!=0)
                	  {
                		  DecodeCycles = clock()-DecodeCycles;
                	  }

                	  OPFDecoderCycles[DecodeCnt]+=DecodeCycles;


                	  if( ((DecWIndex>DecRIndex)&&((DecWIndex-DecRIndex)<=DECODE_SIZE))||
                	      ((DecWIndex<DecRIndex)&&((DecRIndex-DecWIndex)>=DECODE_SIZE))
						)
                	  {

                     	 DecodeCycles = clock();

                     	 break;
                	  }

                	  if(DecodeCycles!=0)
                	  {
                		  DecoderMIPS = (float)OPFDecoderCycles[DecodeCnt]*0.048/(DECODE_SIZE/2);
                		  DecodeCnt++;
                		  DecodeCnt=DecodeCnt%OPF_CNT_SIZE;
                		  DecodeCycles=0;
                		  OPFDecoderCycles[DecodeCnt]=0;
                	  }

              		if(FWReInit==1)
              		{
              			Init_FIR_IIR(ACCUsageModelNext);
              			FWReInit = 0;
              			ACCUsageModel = ACCUsageModelNext;
              		}

              		volatile int temp1, temp2;

              		/* Wait till next block of data is received */
              		if(BlockReady==true)
              		{
              			OPFCnt++;

              			OPFCnt=OPFCnt%OPF_CNT_SIZE;

              			temp1 = clock();

             			OPFTotalCycles[OPFCnt] = OPFCycles - temp1;

             			OPFCycles = temp1;

						/* Clear BlockReady flag for next iteration */
						BlockReady = false;

             			temp2 = clock();

						/* Process current block */
						Process_Block();

						temp2 = clock() - temp2;

						OPFPPCycles[OPFCnt] = temp2;
              		}
                  }

#endif

                  cycles1=clock();
                  vorbis_synthesis_read(&vd,bout); /* tell libvorbis how
                                                      many samples we
                                                      actually consumed */
                  cycles1=clock()-cycles1;
                  cycles1_per_frame+=cycles1;
                  PRINTC("\n\nvorbis_synthesis_read cycles1=%d",cycles1);
                }
              }
            }
            if(ogg_page_eos(&og))eos=1;
          }
        }
        if(!eos){
         cycles1=clock();
          buffer=ogg_sync_buffer(&oy,4096);
#if 0
    bytes=fread(buffer,1,4096,stdin1);

#else
    if(FILE_SIZE-ReadCnt>4096)
    {
    	bytes = 4096;
    	memcpy((void *)buffer, (const void *)pReadPointer, bytes);
    	pReadPointer+=4096;
    	ReadCnt+=4096;
    }
    else
    {
    	bytes = FILE_SIZE-ReadCnt;
    	memcpy((void *)buffer, (const void *)pReadPointer, bytes);
    }
#endif
          ogg_sync_wrote(&oy,bytes);
          if(bytes==0)eos=1;
          cycles1=clock()-cycles1;
           PRINTC("\nfread cycles1=%d",cycles1);
        }
      }

      /* ogg_page and ogg_packet structs always point to storage in
         libvorbis.  They're never freed or manipulated directly */

      vorbis_block_clear(&vb);
      vorbis_dsp_clear(&vd);
    }else{
      fprintf(stderr,"Error: Corrupt header during playback initialization.\n");
    }

    /* clean up this logical bitstream; before exit we see if we're
       followed by another [chained] */

    ogg_stream_clear(&os);
    vorbis_comment_clear(&vc);
    vorbis_info_clear(&vi);  /* must be called last */
  }

  /* OK, clean up the framer */
  ogg_sync_clear(&oy);

  fclose(stdin1);
  fprintf(stderr,"Done.\n");

  return;
}
