#ifndef __UART_H__
#define __UART_H__

#include <stdlib.h>
#include <sys/platform.h>
#include <sys/adi_core.h>
#include <drivers/uart/adi_uart.h>
#include <services/spu/adi_spu.h>
#include "adi_initialize.h"
#include "string.h"
//#include <cdef2156x_rom.h>
//#include <def2156x_rom.h>
#define SAMPLE_RATE 0.0441
#define BLOCK_SIZEE  256

/* Fixed buffer size for the UART RX and TX Message */
#define BUFF_SIZE   100u

/* Set UART Device Number to the required UART to be tested
 *  UART0 -- ADI_UART_0
 *  UART1 -- ADI_UART_1
 *  UART2 -- ADI_UART_2
 */
#define ADI_UART_DEVNUM  ADI_UART_0

/* In Stop DMA mode it is similar to a single descriptor DMA mode so DMA_NUM_DESC should always be one */
#define DMA_NUM_DESC_UART            1u
#define DESC_BUFFER_SIZE        	BUFF_SIZE
#define DMA_MSIZE_IN_BYTES      	1u

/* Macro for reporting errors */
#define REPORT_ERROR        printf

#define SUCCESS   0
#define FAILED   -1

void UartInit(void);

/* structrue to store MIPS info */
typedef struct MIPSInfo
{
	float CoreMIPS;
	float FIRAMIPS;
	float IIRAMIPS;
	float CoreMIPSSaving;
	float Latency;
} MIPSInfo;

/* Enum for ACC usage model information */
typedef enum ACC_USAGE_MODEL
{
	CORE_ONLY,
	DIRECT_REPLACEMENT,
	SPLIT_TASK,
	DATA_PIPELINING,
} ACC_USAGE_MODEL;

/* Struture to store IIR equalizer settings */
typedef struct IIRInfo
{
	char msg[10];
	int  BypassProcessing;
	int IIRGains[5];
}
IIRInfo;

/* These variables are declared as externs as they will be used by the main application to communicate/synchroize with the UART framework */
extern MIPSInfo CoreOnlyMIPS;
extern MIPSInfo DirectReplacementMIPS;
extern MIPSInfo PipelinedMIPS;
extern MIPSInfo SplitTaskMIPS;
extern volatile int cycle_count;
extern volatile int cycles_array[100];
extern volatile int clock_stop;
extern volatile int clock_start;
extern volatile int cycles;
extern volatile int core_cycles;
extern volatile int fira_cycles;
extern volatile int iira_cycles;
extern volatile int FWReInit;
extern volatile int ChangeUseCase;
extern ACC_USAGE_MODEL ACCUsageModel;
extern ACC_USAGE_MODEL ACCUsageModelNext;
extern ADI_UART_HANDLE hUART;
extern float DecoderMIPS;

#if 1
/* Macro to start cycle count */
#define START_CYCLE_COUNT\
		clock_start = clock();\
		core_cycles=0;\
		fira_cycles=0;\
		iira_cycles=0;

/* Macro to take cycle snapshot */
#define TAKE_CYCLE_SNAPSHOT\
	clock_stop = clock();\
	cycles = clock_stop - clock_start;\
	clock_start = clock_stop;

/* Macro to add the recent cycle snapshot to overall core cycles */
#define ADD_TO_CORE_CYCLES\
		core_cycles+=cycles;

/* Macro to add the recent cycle snapshot to overall FIRA cycles */
#define ADD_TO_FIRA_CYCLES\
		fira_cycles+=cycles;

/* Macro to add the recent cycle snapshot to overall IIRA cycles */
#define ADD_TO_IIRA_CYCLES\
		iira_cycles+=cycles;
#else
/* Macro to start cycle count */
#define START_CYCLE_COUNT

/* Macro to take cycle snapshot */
#define TAKE_CYCLE_SNAPSHOT

/* Macro to add the recent cycle snapshot to overall core cycles */
#define ADD_TO_CORE_CYCLES

/* Macro to add the recent cycle snapshot to overall FIRA cycles */
#define ADD_TO_FIRA_CYCLES

/* Macro to add the recent cycle snapshot to overall IIRA cycles */
#define ADD_TO_IIRA_CYCLES

#endif

#if 1
/* Macro to calculate MIPS information to send to PC */
#define CALCULATE_MIPS\
		if(ACCUsageModel==CORE_ONLY)\
		{\
			CoreOnlyMIPS.CoreMIPS=DecoderMIPS+((float)core_cycles)*SAMPLE_RATE/BLOCK_SIZEE;\
			CoreOnlyMIPS.FIRAMIPS=((float)fira_cycles)*SAMPLE_RATE/BLOCK_SIZEE;\
			CoreOnlyMIPS.IIRAMIPS=((float)iira_cycles)*SAMPLE_RATE/BLOCK_SIZEE;\
			CoreOnlyMIPS.CoreMIPSSaving=0.0;\
			CoreOnlyMIPS.Latency=0.0;\
		}\
		else if(ACCUsageModel==DIRECT_REPLACEMENT)\
		{\
			DirectReplacementMIPS.CoreMIPS=DecoderMIPS+((float)core_cycles)*SAMPLE_RATE/BLOCK_SIZEE;\
			DirectReplacementMIPS.FIRAMIPS=((float)fira_cycles)*SAMPLE_RATE/BLOCK_SIZEE;\
			DirectReplacementMIPS.IIRAMIPS=((float)iira_cycles)*SAMPLE_RATE/BLOCK_SIZEE;\
			DirectReplacementMIPS.CoreMIPSSaving=CoreOnlyMIPS.CoreMIPS-DirectReplacementMIPS.CoreMIPS;\
			DirectReplacementMIPS.Latency=0.0;\
		}\
		else if(ACCUsageModel==SPLIT_TASK)\
		{\
			SplitTaskMIPS.CoreMIPS=DecoderMIPS+((float)core_cycles)*SAMPLE_RATE/BLOCK_SIZEE;\
			SplitTaskMIPS.FIRAMIPS=((float)fira_cycles)*SAMPLE_RATE/BLOCK_SIZEE;\
			SplitTaskMIPS.IIRAMIPS=((float)iira_cycles)*SAMPLE_RATE/BLOCK_SIZEE;\
			SplitTaskMIPS.CoreMIPSSaving=CoreOnlyMIPS.CoreMIPS-SplitTaskMIPS.CoreMIPS;\
			SplitTaskMIPS.Latency=0.0;\
		}\
		else if(ACCUsageModel==DATA_PIPELINING)\
		{\
			PipelinedMIPS.CoreMIPS=DecoderMIPS+((float)core_cycles)*SAMPLE_RATE/BLOCK_SIZEE;\
			PipelinedMIPS.FIRAMIPS=((float)fira_cycles)*SAMPLE_RATE/BLOCK_SIZEE;\
			PipelinedMIPS.IIRAMIPS=((float)iira_cycles)*SAMPLE_RATE/BLOCK_SIZEE;\
			PipelinedMIPS.CoreMIPSSaving=CoreOnlyMIPS.CoreMIPS-PipelinedMIPS.CoreMIPS;\
			PipelinedMIPS.Latency=(float)BLOCK_SIZEE/(SAMPLE_RATE*1000);\
		}
#else
#define CALCULATE_MIPS\
		if(ACCUsageModel==CORE_ONLY)\
		{\
			CoreOnlyMIPS.CoreMIPS=DecoderMIPS;\
			CoreOnlyMIPS.FIRAMIPS=((float)fira_cycles)*SAMPLE_RATE/BLOCK_SIZEE;\
			CoreOnlyMIPS.IIRAMIPS=((float)iira_cycles)*SAMPLE_RATE/BLOCK_SIZEE;\
			CoreOnlyMIPS.CoreMIPSSaving=0.0;\
			CoreOnlyMIPS.Latency=0.0;\
		}\
		else if(ACCUsageModel==DIRECT_REPLACEMENT)\
		{\
			DirectReplacementMIPS.CoreMIPS=DecoderMIPS+((float)core_cycles)*SAMPLE_RATE/BLOCK_SIZEE;\
			DirectReplacementMIPS.FIRAMIPS=((float)fira_cycles)*SAMPLE_RATE/BLOCK_SIZEE;\
			DirectReplacementMIPS.IIRAMIPS=((float)iira_cycles)*SAMPLE_RATE/BLOCK_SIZEE;\
			DirectReplacementMIPS.CoreMIPSSaving=CoreOnlyMIPS.CoreMIPS-DirectReplacementMIPS.CoreMIPS;\
			DirectReplacementMIPS.Latency=0.0;\
		}\
		else if(ACCUsageModel==SPLIT_TASK)\
		{\
			SplitTaskMIPS.CoreMIPS=DecoderMIPS+((float)core_cycles)*SAMPLE_RATE/BLOCK_SIZEE;\
			SplitTaskMIPS.FIRAMIPS=((float)fira_cycles)*SAMPLE_RATE/BLOCK_SIZEE;\
			SplitTaskMIPS.IIRAMIPS=((float)iira_cycles)*SAMPLE_RATE/BLOCK_SIZEE;\
			SplitTaskMIPS.CoreMIPSSaving=CoreOnlyMIPS.CoreMIPS-SplitTaskMIPS.CoreMIPS;\
			SplitTaskMIPS.Latency=0.0;\
		}\
		else if(ACCUsageModel==DATA_PIPELINING)\
		{\
			PipelinedMIPS.CoreMIPS=DecoderMIPS+((float)core_cycles)*SAMPLE_RATE/BLOCK_SIZEE;\
			PipelinedMIPS.FIRAMIPS=((float)fira_cycles)*SAMPLE_RATE/BLOCK_SIZEE;\
			PipelinedMIPS.IIRAMIPS=((float)iira_cycles)*SAMPLE_RATE/BLOCK_SIZEE;\
			PipelinedMIPS.CoreMIPSSaving=CoreOnlyMIPS.CoreMIPS-PipelinedMIPS.CoreMIPS;\
			PipelinedMIPS.Latency=(float)BLOCK_SIZEE/(SAMPLE_RATE*1000);\
		}
#endif

#define Switch_APP(BootStreamStart)\
	*pREG_SPU0_SECUREP110= 0x3;\
	*pREG_SPU0_SECUREP111= 0x3;\
	*pREG_SPU0_SECUREP112= 0x3;\
	*pREG_SPU0_SECUREP113= 0x3;\
	*pREG_SPU0_SECUREP108= 0x3;\
	*pREG_SPU0_SECUREP109= 0x3;\
	*pREG_SPU0_SECUREP119= 0x3;\
	*pREG_SHL1C0_CFG2|=0xF00 ;\
	*pREG_SHL1C0_RANGE_START4 = 0x200FE000 ;\
	*pREG_SHL1C0_RANGE_END4 = 0x200FFFFF ;\
	*pREG_SHL1C0_RANGE_START5 = 0x60000000;\
	*pREG_SHL1C0_RANGE_END5 = 0x6FFFFFFF;\
	asm("#include <sys/def215xx_core.h>");\
    asm("IRPTL = 0;");\
    asm("MMASK = BITM_REGF_MMASK_BR8|BITM_REGF_MMASK_BR0|BITM_REGF_MMASK_IRPTEN|BITM_REGF_MMASK_ALUSAT|BITM_REGF_MMASK_TRUNCATE|BITM_REGF_MMASK_PEYEN|BITM_REGF_MMASK_BDCST9|BITM_REGF_MMASK_BDCST1;");\
    asm("IMASK = BITM_REGF_IMASK_EMUI|BITM_REGF_IMASK_RSTI|BITM_REGF_IMASK_PARI|BITM_REGF_IMASK_ILOPI|BITM_REGF_IMASK_CB7I|BITM_REGF_IMASK_IICDI|BITM_REGF_IMASK_SOVFI| BITM_REGF_IMASK_ILADI; nop;");\
    asm("B7 = 0x200FF400;\
   		I7 = ((0x200FF400 + 0xc00 - 1)-((0x200FF400 + 0xc00 - 1) % 4) );\
    	L7 = (( 0xc00 - ((0x200FF400 + 0xc00 - 1) % 4) )/4);\
    	B6 = B7;\
    	I6 = I7; \
        L6 = L7;");\
  adi_rom_Boot(BootStreamStart,0,0,0,0x00010207);

/* Macro to reset MIPS information */
#define RESET_MIPS\
		core_cycles=0;\
		fira_cycles=0;\
		iira_cycles=0;\
		CALCULATE_MIPS

#endif /* __UARTAUTOBAUD_H__ */
